import pytest


class TestRenice:
    @pytest.mark.complete("renice 1")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("renice -g ")
    def test_2(self, completion):
        assert completion



from shlex import quote

import pytest

from conftest import assert_bash_exec, assert_complete, bash_env_saved


def join(words):
    """Return a shell-escaped string from *words*."""
    return " ".join(quote(word) for word in words)


@pytest.mark.bashcomp(
    cmd=None,
    cwd="_command_offset",
    ignore_env=r"^[+-](COMPREPLY|REPLY)=",
)
class TestUnitCommandOffset:
    wordlist = sorted(["foo", "bar"])

    @pytest.fixture(scope="class")
    def functions(self, bash):
        assert_bash_exec(
            bash,
            "_cmd1() { _comp_command_offset 1; }; complete -F _cmd1 cmd1; "
            "complete -F _comp_command meta; "
            "_compfunc() { COMPREPLY=(%s); }" % join(self.wordlist),
        )

        completions = [
            'complete -F _compfunc "${COMP_WORDS[0]}"',
            'complete -W %s "${COMP_WORDS[0]}"' % quote(join(self.wordlist)),
            'COMPREPLY=(dummy); complete -r "${COMP_WORDS[0]}"',
            "COMPREPLY+=(${#COMPREPLY[@]})",
        ]
        for idx, comp in enumerate(completions, 2):
            assert_bash_exec(
                bash,
                "_cmd%(idx)s() { %(comp)s && return 124; }; "
                "complete -F _cmd%(idx)s cmd%(idx)s"
                % {"idx": idx, "comp": comp},
            )

        assert_bash_exec(
            bash, "complete -W %s 'cmd!'" % quote(join(self.wordlist))
        )
        assert_bash_exec(bash, 'complete -W \'"$word1" "$word2"\' cmd6')

        assert_bash_exec(bash, "complete -C ./completer cmd7")

    def test_1(self, bash, functions):
        assert_complete(bash, 'cmd1 "/tmp/aaa bbb" ')
        assert_bash_exec(bash, "! complete -p aaa", want_output=None)

    @pytest.mark.parametrize(
        "cmd,expected_completion",
        [
            ("cmd2", wordlist),
            ("cmd3", wordlist),
            ("cmd4", []),
            ("cmd5", ["0"]),
        ],
    )
    def test_2(self, bash, functions, cmd, expected_completion):
        """Test meta-completion for completion functions that signal that
        completion should be retried (i.e. change compspec and return 124).

        cmd2: The case when the completion spec is overwritten by the one that
        contains "-F func"

        cmd3: The case when the completion spec is overwritten by the one
        without "-F func".

        cmd4: The case when the completion spec is removed, in which we expect
        no completions.  This mimics the behavior of Bash's progcomp for the
        exit status 124.

        cmd5: The case when the completion spec is unchanged.  The retry should
        be attempted at most once to avoid infinite loops.  COMPREPLY should be
        cleared before the retry.
        """
        assert assert_complete(bash, "meta %s " % cmd) == expected_completion

    @pytest.mark.parametrize(
        "cmd,expected_completion",
        [
            ("cmd7 ", wordlist),
            ("cmd7 l", ["line\\^Jtwo", "long"]),
            ("cmd7 lo", ["ng"]),
            ("cmd7 line", ["\\^Jtwo"]),
            ("cmd7 cont1", ["cont10", "cont11\\"]),
        ],
    )
    def test_3(self, bash, functions, cmd, expected_completion):
        got = assert_complete(bash, f"cmd1 {cmd}")
        assert got == assert_complete(bash, cmd)
        assert got == expected_completion

    def test_cmd_quoted(self, bash, functions):
        assert assert_complete(bash, "meta 'cmd2' ") == self.wordlist

    def test_cmd_specialchar(self, bash, functions):
        assert assert_complete(bash, "meta 'cmd!' ") == self.wordlist

    def test_space(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable("word1", "a b c")
            bash_env.write_variable("word2", "d e f")
            assert assert_complete(bash, "meta cmd6 ") == ["a b c", "d e f"]

    @pytest.fixture(scope="class")
    def find_original_word_functions(self, bash):
        assert_bash_exec(
            bash,
            "_comp_test_reassemble() {"
            "    local IFS=$' \\t\\n' REPLY;"
            '    COMP_LINE=$1; _comp_split COMP_WORDS "$2"; COMP_CWORD=$((${#COMP_WORDS[@]}-1));'
            "    _comp__reassemble_words = words cword;"
            "}",
        )
        assert_bash_exec(
            bash,
            "_comp_test_1() {"
            '    local COMP_WORDS COMP_LINE COMP_CWORD words cword REPLY; _comp_test_reassemble "$1" "$2";'
            '    _comp__find_original_word "$3";'
            '    echo "$REPLY";'
            "}",
        )

    def test_find_original_word_1(self, bash, find_original_word_functions):
        result = assert_bash_exec(
            bash,
            '_comp_test_1 "sudo su do su do abc" "sudo su do su do abc" 3',
            want_output=True,
        ).strip()
        assert result == "3"

    def test_find_original_word_2(self, bash, find_original_word_functions):
        result = assert_bash_exec(
            bash,
            '_comp_test_1 "sudo --prefix=su su do abc" "sudo --prefix = su su do abc" 2',
            want_output=True,
        ).strip()
        assert result == "4"



import pytest


class TestIperf:
    @pytest.mark.complete("iperf ", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("iperf --bind ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("iperf --client foo --", require_cmd=True)
    def test_3(self, completion):
        assert completion
        assert "--daemon" not in completion

    @pytest.mark.complete("iperf --server --", require_cmd=True)
    def test_4(self, completion):
        assert "--daemon" in completion

    @pytest.mark.complete("iperf -", require_cmd=True)
    def test_5(self, completion):
        assert completion

    @pytest.mark.complete("iperf --format ", require_cmd=True)
    def test_format(self, completion):
        # 2.0.5 has only up to m/M, later may have g/G, t/T, ...
        assert all(x in completion for x in "k m K M".split())



import sys

import pytest


class TestService:
    @pytest.mark.xfail(
        sys.platform == "darwin",
        reason="Service completion not available on macOS",
    )
    @pytest.mark.complete("service ")
    def test_1(self, completion):
        assert completion



import pytest


class TestLn:
    @pytest.mark.complete("ln ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ln -", require_longopt=True)
    def test_options(self, completion):
        assert completion



import pytest


class TestMr:
    @pytest.mark.complete("mr ")
    def test_1(self, completion):
        assert completion

    # man -h tests below: Some mr versions require man to be around in order
    # to provide useful output.

    @pytest.mark.complete(
        "mr --", require_cmd=True, xfail="! man -h &>/dev/null"
    )
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete(
        "mr -c shared/default/foo.d/", xfail="! man -h &>/dev/null"
    )
    def test_3(self, completion):
        assert completion == "foo"

    @pytest.mark.complete(
        "mr bootstrap shared/default/",
        require_cmd=True,
        xfail="! man -h &>/dev/null",
    )
    def test_4(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete(
        "mr clean -",
        require_cmd=True,
        xfail="! man -h &>/dev/null",
        # "clean" does not exist before mr 1.20141023
        skipif="! mr help 2>&1 | command grep -qwF clean",
    )
    def test_5(self, completion):
        assert completion == "f"

    @pytest.mark.complete(
        "mr commit -", require_cmd=True, xfail="! man -h &>/dev/null"
    )
    def test_6(self, completion):
        assert completion == "m"

    @pytest.mark.complete(
        "mr status ", require_cmd=True, xfail="! man -h &>/dev/null"
    )
    def test_7(self, completion):
        assert not completion

    @pytest.mark.complete(
        "mr run ", require_cmd=True, xfail="! man -h &>/dev/null"
    )
    def test_8(self, completion):
        assert completion



import pytest


class TestWine:
    @pytest.mark.complete("wine ", cwd="shared/default")
    def test_1(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]

    @pytest.mark.complete("wine notepad ", cwd="shared/default")
    def test_2(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete("wine --help ")
    def test_no_complete_after_help(self, completion):
        assert not completion



import pytest


class TestHelp:
    @pytest.mark.complete("help ")
    def test_basic(self, completion):
        assert "help" in completion

    @pytest.mark.complete("help -")
    def test_options(self, completion):
        assert completion

    @pytest.mark.complete(
        r"help \(",
        skipif="! compgen -A helptopic | grep -qxF '(( ... ))'",  # bash 4.2
    )
    def test_parens(self, completion):
        # Assumption: an item like "(( ... ))" exists in the output
        assert any(
            x.startswith(r"\(") and x.endswith(r"\)\)") for x in completion
        )



import pytest


class TestDmypy:
    @pytest.mark.complete(
        "dmypy ", require_cmd=True, xfail="! dmypy --help &>/dev/null"
    )
    def test_commands(self, completion):
        assert "help" in completion
        assert not any("," in x for x in completion)

    @pytest.mark.complete("dmypy -", require_longopt=True)
    def test_options(self, completion):
        assert "--help" in completion



import pytest


class TestDpkg:
    @pytest.mark.complete("dpkg --c", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("dpkg -L ", xfail='test ! "$(dpkg -l 2>/dev/null)"')
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("dpkg -i ~")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("dpkg -i dpkg/")
    def test_i_deb(self, completion):
        assert completion == "bash-completion-test-subject.deb"

    @pytest.mark.complete("dpkg -")
    def test_no_trailing_dash_options(self, completion):
        assert not any(x.endswith("-") for x in completion)



import pytest


@pytest.mark.bashcomp(
    cmd="ssh-keyscan",
)
class TestSshKeyscan:
    @pytest.mark.complete("ssh-keyscan ")
    def test_basic(self, completion):
        assert completion

    @pytest.mark.complete("ssh-keyscan -", require_cmd=True)
    def test_options(self, completion):
        assert completion

    @pytest.mark.complete("ssh-keyscan -t rsa,", require_cmd=True)
    def test_type_delimited(self, completion):
        assert completion
        assert "rsa" not in completion



import pytest


@pytest.mark.bashcomp(cmd="mii-tool")
class TestMiiTool:
    @pytest.mark.complete("mii-tool ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mii-tool -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestHcitool:
    @pytest.mark.complete("hcitool ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("hcitool -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestLrzip:
    @pytest.mark.complete("lrzip ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lrzip ~")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("lrzip -", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete(
        "lrzip --",
        # require_longopt not applicable, useful --help may give nonzero exit
        require_cmd=True,
        xfail=("! { lrzip --help 2>&1 || :; } | command grep -qF -- --help"),
    )
    def test_longopt(self, completion):
        assert completion



import pytest

from conftest import assert_bash_exec, assert_complete, bash_env_saved


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+COMPREPLY=")
class TestUtilCompgenCommands:
    @pytest.fixture(scope="class")
    def functions(self, request, bash):
        assert_bash_exec(
            bash,
            r"_comp_compgen_commands__test() {"
            r"    local COMPREPLY=() cur=${1-};"
            r"    _comp_compgen_commands;"
            r'    printf "%s\n" "${COMPREPLY[@]-}";'
            r"}",
        )
        assert_bash_exec(
            bash,
            "_comp_cmd_ccc() {"
            "    local cur;"
            "    _comp_get_words cur;"
            "     unset -v COMPREPLY;"
            "    _comp_compgen_commands;"
            "}; complete -F _comp_cmd_ccc ccc",
        )

    def test_basic(self, bash, functions):
        output = assert_bash_exec(
            bash, "_comp_compgen_commands__test sh", want_output=True
        )
        assert output.strip()

    @pytest.mark.parametrize(
        "shopt_no_empty,result_empty", ((True, True), (False, False))
    )
    def test_empty(self, bash, functions, shopt_no_empty, result_empty):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("no_empty_cmd_completion", shopt_no_empty)
            output = assert_bash_exec(
                bash, "_comp_compgen_commands__test", want_output=True
            )
        assert (output.strip() == "") == result_empty

    def test_spaces(self, bash, functions):
        completion = assert_complete(bash, "ccc shared/default/bar")
        assert completion == r"\ bar.d/"



import pytest


class TestIp:
    @pytest.mark.complete("ip ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ip a ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("ip route replace ", require_cmd=True)
    def test_r_r(self, completion):
        assert completion

    @pytest.mark.complete(
        "ip stats show group ",
        require_cmd=True,
        skipif="ip stats help 2>/dev/null; (( $? != 255 ))",
    )
    def test_stats(self, completion):
        # "link" was one of the first groups added, should always be there
        assert "link" in completion

    @pytest.mark.complete(
        "ip neigh show nud ",
        require_cmd=True,
        skipif="ip neigh help 2>&1 | grep 'STATE :=' > /dev/null; (( $? != 0 ))",
    )
    def test_neigh_state(self, completion):
        assert "stale" in completion

    @pytest.mark.complete(
        "ip monitor ",
        require_cmd=True,
        skipif="ip monitor help 2>/dev/null; (( $? != 255 ))",
    )
    def test_monitor(self, completion):
        assert "neigh" in completion
        assert "all" in completion

    @pytest.mark.complete("ip netconf ")
    def test_netconf(self, completion):
        assert "show" in completion

    @pytest.mark.complete("ip link property add ")
    def test_link_property(self, completion):
        assert "altname" in completion
        assert "dev" in completion

    @pytest.mark.complete(
        "ip addr show type ",
        require_cmd=True,
        skipif="ip link help 2>/dev/null; (( $? != 255 ))",
    )
    def test_addr_type(self, completion):
        assert "bridge" in completion

    @pytest.mark.complete("ip -", require_cmd=True)
    def test_options(self, completion):
        assert "-family" in completion



import pytest


class TestFio:
    @pytest.mark.complete("fio ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("fio --", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("fio --debug=foo,", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("fio --ioengin", require_cmd=True)
    def test_cmdhelp_all(self, completion):
        """Test we got a "known present" option from --cmdhelp=all."""
        assert completion == "e=" or "e" in completion

    @pytest.mark.complete("fio --ioengine=", require_cmd=True)
    def test_enghelp(self, completion):
        """Test --enghelp parsing."""
        assert completion
        assert all(x == x.strip() for x in completion)
        assert all(")" not in x for x in completion)

    @pytest.mark.complete("fio --unlink=", require_cmd=True)
    def test_cmdhelp_boolean(self, completion):
        """Test --cmdhelp=COMMAND boolean parsing."""
        assert completion == "0 1".split()

    @pytest.mark.complete("fio --kb_base=", require_cmd=True)
    def test_cmdhelp_valid_values(self, completion):
        """Test --cmdhelp=COMMAND valid values parsing."""
        # We expect kb_base args to be stable, no additions/removals.
        assert completion == "1000 1024".split()

    @pytest.mark.complete("fio --non_exist3nt_option=", require_cmd=True)
    def test_cmdhelp_nonexistent(self, completion):
        """Test --cmdhelp=COMMAND errors out gracefully."""
        assert not completion

    @pytest.mark.complete(
        "fio --crctest=",
        require_cmd=True,
        xfail="! fio --help 2>&1 | command grep -q -- --crctest",
    )
    def test_crctest(self, completion):
        assert "sha1" in completion

    @pytest.mark.complete("fio --debug=", require_cmd=True)
    def test_debug(self, completion):
        assert "process" in completion



import pytest


class TestLilo:
    @pytest.mark.complete("lilo -")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lilo -C lilo/lilo.conf -D ")
    def test_labels(self, completion):
        # Note that 2.4.33 should not be here, it's commented out
        assert completion == sorted("try tamu PCDOS WinXP oldDOS".split())

    @pytest.mark.complete("lilo -C -D ")
    def test_labels_incorrect_command(self, completion):
        assert not completion



import os

import pytest


class TestValgrind:
    # b: Assume we have at least bash that starts with b in PATH
    @pytest.mark.complete("valgrind b")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("valgrind -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("valgrind --tool=memche", require_cmd=True)
    def test_3(self, completion):
        assert completion == "ck" or "--tool=memcheck" in completion

    @pytest.mark.complete(
        "valgrind --tool=helgrind --history-l", require_cmd=True
    )
    def test_4(self, completion):
        assert completion == "evel=" or "--history-level=" in completion
        assert not completion.endswith(" ")

    @pytest.mark.complete(r"valgrind --log-file=v\ 0.log ./bin/", cwd="shared")
    def test_5(self, completion):
        expected = sorted(
            [
                "%s/"
                for x in os.listdir("shared/bin")
                if os.path.isdir("shared/bin/%s" % x)
            ]
            + [
                x
                for x in os.listdir("shared/bin")
                if os.path.isfile("shared/bin/%s" % x)
            ]
        )
        assert completion == expected



import pytest


class TestLzop:
    @pytest.mark.complete("lzop ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lzop ~")
    def test_2(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(pre_cmds=("INFOPATH+=:$PWD/info:",))
class TestPinfo:
    @pytest.mark.complete("pinfo -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("pinfo bash")
    def test_2(self, completion):
        assert completion



import pytest


class TestDot:
    @pytest.mark.complete("dot ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("dot t", cwd="dot")
    def test_2(self, completion):
        assert completion == ["test1.gv", "test2.dot"]

    @pytest.mark.complete("dot test1", cwd="dot")
    def test_3(self, completion):
        assert completion == ".gv"



import pytest


@pytest.mark.bashcomp(ignore_env=r"^\+CDPATH=$")
class TestCd:
    @pytest.mark.complete("cd shared/default/")
    def test_1(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]

    @pytest.mark.complete("cd foo", env=dict(CDPATH="shared/default"))
    def test_2(self, completion):
        assert completion == ".d/"

    @pytest.mark.complete("cd foo")
    def test_3(self, completion):
        assert not completion

    @pytest.mark.complete(
        "cd ", cwd="shared/default/foo.d", env=dict(CDPATH="")
    )
    def test_4(self, completion):
        assert not completion  # No subdirs nor CDPATH

    @pytest.mark.complete("cd shared/default/", trail="foo")
    def test_dir_at_point(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]

    @pytest.mark.complete("cd -")
    def test_options(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(
    cmd="ssh-copy-id",
    pre_cmds=(
        # Some old versions of ssh-copy-id won't output even usage if no
        # identities are found. Try to make sure there is at least one.
        "HOME=$PWD/ssh-copy-id",
    ),
    ignore_env=r"^[+-]_comp_cmd_scp__path_esc=",
)
class TestSshCopyId:
    @pytest.mark.complete("ssh-copy-id -", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest


class TestCpio:
    @pytest.mark.complete("cpio --")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("cpio -R ")
    def test_2(self, bash, completion, output_sort_uniq):
        users = output_sort_uniq("compgen -u")
        assert completion == users



import pytest

from conftest import assert_bash_exec


class TestGcc:
    @pytest.fixture(scope="class")
    def gcc_with_completion(self, bash):
        got = assert_bash_exec(
            bash, "gcc --help=common || :", want_output=True
        )
        if "--completion" not in got:
            pytest.skip("GCC does not support --completion")

    @pytest.fixture(scope="class")
    def gcc_x86(self, bash):
        got = assert_bash_exec(bash, "gcc -v || :", want_output=True)
        if "Target: x86" not in got:
            pytest.skip("Not a x86 GCC")

    @pytest.mark.complete("gcc ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("gcc -fsanitize=add")
    def test_enum_value(self, completion, gcc_with_completion):
        assert completion == "ress"

    @pytest.mark.complete("gcc -fsanitize=")
    def test_enum_value_with_eq(self, completion, gcc_with_completion):
        assert "address" in completion

    @pytest.mark.complete("gcc -fno-ipa-ic")
    def test_negative_option(self, completion, gcc_with_completion):
        assert "-fno-ipa-icf" in completion

    @pytest.mark.complete("gcc -fxyz-abc")
    def test_no_completion(self, completion):
        assert not completion

    @pytest.mark.complete("gcc --param ")
    def test_param_with_space(self, completion, gcc_with_completion):
        assert len(completion) > 50
        # starting with GCC 10.1 param end with =
        assert (
            "lto-partitions" in completion or "lto-partitions=" in completion
        )

    @pytest.mark.complete("gcc --param=lto-max-p")
    def test_param_with_eq(self, completion, gcc_with_completion):
        # starting with GCC 10.1 param ends with =
        assert completion in ("artition", "artition=")

    @pytest.mark.complete("gcc -march=amd")
    def test_march_amd(self, completion, gcc_with_completion, gcc_x86):
        assert completion == "fam10"

    @pytest.mark.complete("gcc -march=")
    def test_march(self, completion, gcc_with_completion):
        # https://github.com/scop/bash-completion/issues/1201
        # native: x86, some others
        # power*: ppc
        assert "native" in completion or any(
            x.startswith("power") for x in completion
        )

    @pytest.mark.complete("gcc -mtune=")
    def test_mtune(self, completion, gcc_with_completion):
        # https://github.com/scop/bash-completion/issues/1201
        # generic: x86, some others
        # power*: ppc
        assert "generic" in completion or any(
            x.startswith("power") for x in completion
        )



import pytest

ENV = dict(PATH="$PWD/xrandr:$PATH")
OUTPUTS = sorted("DP-0 DP-1 DP-2 DP-3 eDP-1-1 HDMI-0".split())


@pytest.mark.bashcomp(pre_cmds=("PATH=$PATH:$PWD/xrandr",))
class TestXrandr:
    @pytest.mark.complete("xrandr ", require_cmd=True)
    def test_no_args(self, completion):
        assert completion

    @pytest.mark.complete("xrandr -", require_cmd=True)
    def test_single_dash(self, completion):
        assert completion

    @pytest.mark.complete("xrandr --output ", env=ENV)
    def test_output(self, completion):
        assert completion == OUTPUTS

    @pytest.mark.complete("xrandr --output HDMI-0 --left-of ", env=ENV)
    def test_output_left_of(self, completion):
        assert completion == OUTPUTS

    @pytest.mark.complete("xrandr --output HDMI-0 --reflect ", env=ENV)
    def test_output_reflect(self, completion):
        assert completion == sorted("normal x y xy".split())

    @pytest.mark.complete("xrandr --reflect ", require_cmd=True)
    def test_output_reflect_nooutput(self, completion):
        assert not completion

    @pytest.mark.complete("xrandr --output HDMI-0 --rotate ", env=ENV)
    def test_output_rotate(self, completion):
        assert completion == sorted("normal inverted left right".split())

    @pytest.mark.complete("xrandr --rotate ", require_cmd=True)
    def test_output_rotate_nooutput(self, completion):
        assert not completion

    @pytest.mark.complete("xrandr --output HDMI-0 --filter ", env=ENV)
    def test_output_filter(self, completion):
        assert completion == sorted("bilinear nearest".split())

    @pytest.mark.complete("xrandr --output HDMI-0 --mode ", env=ENV)
    def test_output_mode(self, completion):
        assert completion == sorted(
            "1024x768 1280x1024 1280x800 1600x900 1920x1080 720x480 "
            "800x600 1152x864 1280x720 1440x900 1680x1050 640x480 720x576".split()
        )

    @pytest.mark.complete("xrandr --mode ", require_cmd=True)
    def test_output_mode_nooutput(self, completion):
        assert not completion

    @pytest.mark.complete("xrandr --addmode ", env=ENV)
    def test_addmode_first(self, completion):
        assert completion == OUTPUTS

    @pytest.mark.complete("xrandr --addmode HDMI-0 ", env=ENV)
    def test_addmode_second(self, completion):
        assert completion == sorted(
            "1024x576 1280x800 1440x900 320x200 432x243 640x350 700x450 800x450 928x696 "
            "1024x768 1280x960 1600x900 320x240 480x270 640x360 700x525 800x600 960x540 "
            "1024x768i 1368x768 1680x1050 360x200 512x288 640x400 720x400 832x624 960x600 "
            "1152x864 1400x1050 1920x1080 360x202 512x384 640x480 720x405 840x525 960x720 "
            "1280x1024 1400x900 320x175 400x300 512x384i 640x512 720x480 864x486 "
            "1280x720 1440x810 320x180 416x312 576x432 684x384 720x576 896x672".split()
        )

    @pytest.mark.complete("xrandr --delmode ", env=ENV)
    def test_delmode_first(self, completion):
        assert completion == OUTPUTS

    @pytest.mark.complete("xrandr --delmode HDMI-0 ", env=ENV)
    def test_delmode_second(self, completion):
        assert completion == sorted(
            "1024x768 1280x1024 1280x800 1600x900 1920x1080 720x480 "
            "800x600 1152x864 1280x720 1440x900 1680x1050 640x480 720x576".split()
        )

    @pytest.mark.complete("xrandr --dpi ", env=ENV)
    def test_dpi(self, completion):
        assert completion == OUTPUTS

    @pytest.mark.complete("xrandr -o ", env=ENV)
    def test_orientation(self, completion):
        assert completion == sorted(
            "normal inverted left right 0 1 2 3".split()
        )

    @pytest.mark.complete("xrandr --setmonitor testmonitor ", env=ENV)
    def test_setmonitor_second(self, completion):
        assert completion == sorted("auto".split())

    @pytest.mark.complete("xrandr --setmonitor testmonitor auto ", env=ENV)
    def test_setmonitor_third(self, completion):
        assert completion == OUTPUTS + ["none"]

    @pytest.mark.complete("xrandr --delmonitor ", env=ENV)
    def test_delmonitor(self, completion):
        assert completion == sorted("eDP-1-1 HDMI-0".split())

    @pytest.mark.complete("xrandr --setprovideroutputsource ", env=ENV)
    def test_setprovideroutputsource_first(self, completion):
        assert completion == sorted("modesetting".split())

    @pytest.mark.complete(
        "xrandr --setprovideroutputsource modesetting ", env=ENV
    )
    def test_setprovideroutputsource_second(self, completion):
        assert completion == sorted("0x0 modesetting NVIDIA-0".split())

    @pytest.mark.complete("xrandr --setprovideroffloadsink ", env=ENV)
    def test_setprovideroffloadsink_first(self, completion):
        assert completion == sorted("modesetting".split())

    @pytest.mark.complete(
        "xrandr --setprovideroffloadsink modesetting ", env=ENV
    )
    def test_setprovideroffloadsink_second(self, completion):
        assert completion == sorted("0x0 modesetting".split())



import pytest

from conftest import assert_complete, partialize


class TestFinger:
    @pytest.fixture(scope="class")
    def users_at(self, bash, output_sort_uniq):
        return output_sort_uniq("compgen -u -S @")

    @pytest.mark.complete("finger ")
    def test_1(self, bash, completion, users_at):
        assert completion == users_at

    @pytest.mark.complete("finger r")
    def test_2(self, bash, completion, users_at):
        if not any(x.startswith("r") for x in users_at):
            pytest.skip("No users starting with r")
        assert completion
        idx = 1 if len(completion) == 1 else 0
        assert completion == sorted(
            x[idx:] for x in users_at if x.startswith("r")
        )
        assert not completion.endswith(" ")

    def test_partial_hostname(self, bash, known_hosts):
        first_char, partial_hosts = partialize(bash, known_hosts)
        user = "test"
        completion = assert_complete(bash, f"finger {user}@{first_char}")
        if len(completion) == 1:
            assert completion == partial_hosts[0][1:]
        else:
            assert completion == [f"{user}@{x}" for x in partial_hosts]



import pytest


class TestJavac:
    @pytest.mark.complete("javac ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("javac -cp java/")
    def test_2(self, completion):
        assert completion == "a/ bashcomp.jar".split()



import pytest


@pytest.mark.bashcomp(cmd="update-alternatives")
class TestUpdateAlternatives:
    @pytest.mark.complete("update-alternatives --", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("update-alternatives --", require_cmd=True)
    def test_2(self, bash, completion):
        assert all(x in completion for x in ["--list", "--remove"])



import pytest


class TestProtoc:
    @pytest.mark.complete("protoc ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("protoc -", require_cmd=True)
    def test_2(self, completion):
        assert completion
        assert any(
            x.endswith("_out") or x.endswith("_out=") for x in completion
        )

    @pytest.mark.complete(
        "protoc --non_existent_plugin_out ", cwd="shared/default"
    )
    def test_all_out(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]



import pytest


class TestDeclare:
    @pytest.mark.complete("declare -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("declare +", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("declare -p BASH_ARG")
    def test_3(self, completion):
        # bash 5.0 has BASH_ARGV0 too
        assert all(x in completion for x in "BASH_ARGC BASH_ARGV".split())

    @pytest.mark.complete("declare -f _comp_comp")
    def test_4(self, completion):
        assert "_comp_compgen" in completion

    @pytest.mark.complete("declare -a BASH_VERS")
    def test_arrayvar(self, completion):
        assert "INFO" in completion

    @pytest.mark.complete("declare -f BASH_VERS")
    def test_no_arrayvar_for_f(self, completion):
        assert "INFO" not in completion

    @pytest.mark.complete("declare -i BASH_VERS")
    def test_no_arrayvar_for_i(self, completion):
        assert "INFO" not in completion



import pytest


class TestScreen:
    @pytest.mark.complete("screen -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("screen -c shared/default/")
    def test_2(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete("screen cat ")
    def test_3(self, completion):
        assert completion

    # Assume at least vt100 and friends are there
    @pytest.mark.complete("screen -T vt")
    def test_4(self, completion):
        assert completion

    @pytest.mark.complete("screen -T foo ca")
    def test_5(self, completion):
        assert completion == "t" or "cat" in completion

    @pytest.mark.complete("screen //")
    def test_telnet(self, completion):
        assert completion == "telnet"

    @pytest.mark.complete("screen cat //")
    def test_not_telnet(self, completion):
        assert completion != "telnet"

    @pytest.mark.complete("screen //telnet ", env=dict(HOME="$PWD/shared"))
    def test_telnet_first_arg(self, completion):
        assert "bash-completion-canary-host.local" in completion

    @pytest.mark.complete("screen //telnet foo ", env=dict(HOME="$PWD/shared"))
    def test_telnet_other_args(self, completion):
        assert not completion

    @pytest.mark.complete("screen /dev/ttyUSB0 ")
    def test_serial_2nd_arg(self, completion):
        assert "19200" in completion

    @pytest.mark.complete("screen /dev/ttyUSB0 9600 ")
    def test_serial_3rdplus_arg(self, completion):
        assert not completion



import pytest


class TestMcrypt:
    @pytest.mark.complete("mcrypt ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mcrypt -a ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("mcrypt -m ", require_cmd=True)
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("mcrypt -", require_cmd=True)
    def test_4(self, completion):
        assert completion



import pytest


class TestBadblocks:
    @pytest.mark.complete("badblocks ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("badblocks -", require_cmd=True)
    def test_2(self, completion):
        assert completion
        assert all(x not in completion for x in "-w -X".split())



import pytest

from conftest import assert_bash_exec, assert_complete


class TestSudo:
    @pytest.mark.complete("sudo -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("sudo cd foo", cwd="shared/default")
    def test_2(self, completion):
        assert completion == ".d/"
        assert not completion.endswith(" ")

    @pytest.mark.complete("sudo sh share")
    def test_3(self, completion):
        assert completion == "d/"
        assert not completion.endswith(" ")

    @pytest.mark.complete("sudo mount /dev/sda1 def", cwd="shared")
    def test_4(self, completion):
        assert completion == "ault/"
        assert not completion.endswith(" ")

    @pytest.mark.complete("sudo -e -u root bar foo", cwd="shared/default")
    def test_5(self, completion):
        assert completion == "foo foo.d/".split()

    def test_6(self, bash, part_full_user):
        part, full = part_full_user
        completion = assert_complete(bash, "sudo chown %s" % part)
        assert completion == full[len(part) :]
        assert completion.endswith(" ")

    def test_7(self, bash, part_full_user, part_full_group):
        _, user = part_full_user
        partgroup, fullgroup = part_full_group
        completion = assert_complete(
            bash, "sudo chown %s:%s" % (user, partgroup)
        )
        assert completion == fullgroup[len(partgroup) :]
        assert completion.endswith(" ")

    def test_8(self, bash, part_full_group):
        part, full = part_full_group
        completion = assert_complete(bash, "sudo chown dot.user:%s" % part)
        assert completion == full[len(part) :]
        assert completion.endswith(" ")

    @pytest.mark.parametrize(
        "prefix",
        [
            r"funky\ user:",
            "funky.user:",
            r"funky\.user:",
            r"fu\ nky.user:",
            r"f\ o\ o\.\bar:",
            r"foo\_b\ a\.r\ :",
        ],
    )
    def test_9(self, bash, part_full_group, prefix):
        """Test preserving special chars in $prefix$partgroup<TAB>."""
        part, full = part_full_group
        completion = assert_complete(bash, "sudo chown %s%s" % (prefix, part))
        assert completion == full[len(part) :]
        assert completion.endswith(" ")

    def test_10(self, bash, part_full_user, part_full_group):
        """Test giving up on degenerate cases instead of spewing junk."""
        _, user = part_full_user
        partgroup, _ = part_full_group
        for x in range(2, 5):
            completion = assert_complete(
                bash, "sudo chown %s%s:%s" % (user, x * "\\", partgroup)
            )
            assert not completion

    def test_11(self, bash, part_full_group):
        """Test graceful fail on colon in user/group name."""
        part, _ = part_full_group
        completion = assert_complete(bash, "sudo chown foo:bar:%s" % part)
        assert not completion

    def test_12(self, bash):
        assert_complete(bash, 'sudo "/tmp/aaa bbb" ')
        assert_bash_exec(bash, "! complete -p aaa", want_output=None)



import pytest


class TestMysql:
    @pytest.mark.complete("mysql --")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mysql --default-character-set=")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("mysql --non-existent-option=--")
    def test_3(self, completion):
        assert not completion



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(
    cmd=None, cwd="shared", ignore_env=r"^\+declare -f __tester$"
)
class TestUnitAbsPath:
    @pytest.fixture
    def functions(self, bash):
        assert_bash_exec(
            bash,
            (
                "__tester() { "
                "local REPLY; "
                '_comp_abspath "$1"; '
                'printf %s "$REPLY"; '
                "}"
            ),
        )

    def test_non_pollution(self, bash):
        """Test environment non-pollution, detected at teardown."""
        assert_bash_exec(
            bash,
            "foo() { local REPLY=; _comp_abspath bar; }; foo; unset -f foo",
            want_output=None,
        )

    def test_absolute(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /foo/bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/foo/bar"

    def test_relative(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester foo/bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip().endswith("/shared/foo/bar")

    def test_cwd1(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester ./foo/./bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip().endswith("/shared/foo/bar")

    def test_cwd2(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /.",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/"

    def test_cwd3(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /foo/.",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/foo"

    def test_cwd4(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /././.",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/"

    def test_parent1(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester ../shared/foo/bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip().endswith(
            "/shared/foo/bar"
        ) and not output.strip().endswith("../shared/foo/bar")

    def test_parent2(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /foo/..",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/"

    def test_parent3(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /..",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/"

    def test_parent4(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /../foo/bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/foo/bar"

    def test_parent5(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /../../foo/bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/foo/bar"

    def test_parent6(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /foo/../bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/bar"

    def test_parent7(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /foo/../../bar",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/bar"

    def test_parent8(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /dir1/dir2/dir3/../dir4/../../foo",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/dir1/foo"

    def test_parent9(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester //dir1/dir2///../foo",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/dir1/foo"

    def test_parent10(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /dir1/dir2/dir3/..",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/dir1/dir2"

    def test_parent11(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /dir1/dir2/dir3/../..",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/dir1"

    def test_parent12(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "__tester /dir1/dir2/dir3/../../../..",
            want_output=True,
            want_newline=False,
        )
        assert output.strip() == "/"



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(cmd=None)
class TestUtilCompgenSplit:
    @pytest.fixture
    def functions(self, bash):
        assert_bash_exec(
            bash,
            "_comp__test_dump() { ((${#arr[@]})) && printf '<%s>' \"${arr[@]}\"; echo; }",
        )
        assert_bash_exec(
            bash,
            '_comp__test_compgen() { local -a arr=(00); _comp_compgen -v arr "$@"; _comp__test_dump; }',
        )

        assert_bash_exec(
            bash,
            "_comp__test_cmd1() { echo foo bar; echo baz; }",
        )
        assert_bash_exec(
            bash,
            '_comp__test_attack() { echo "\\$(echo should_not_run >&2)"; }',
        )

    def test_1_basic(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "<foo><bar><baz>"

    def test_2_attack(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -- "$(_comp__test_attack)"',
            want_output=True,
        )
        assert output.strip() == "<$(echo><should_not_run><>&2)>"

    def test_3_sep1(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -l -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "<foo bar><baz>"

    def test_3_sep2(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_compgen split -F $'b\\n' -- \"$(_comp__test_cmd1)\"",
            want_output=True,
        )
        assert output.strip() == "<foo ><ar><az>"

    def test_4_optionX(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -X bar -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "<foo><baz>"

    def test_4_optionS(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -S .txt -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "<foo.txt><bar.txt><baz.txt>"

    def test_4_optionP(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -P /tmp/ -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "</tmp/foo></tmp/bar></tmp/baz>"

    def test_4_optionPS(self, bash, functions):
        output = assert_bash_exec(
            bash,
            '_comp__test_compgen split -P [ -S ] -- "$(_comp__test_cmd1)"',
            want_output=True,
        )
        assert output.strip() == "<[foo]><[bar]><[baz]>"

    def test_5_empty(self, bash, functions):
        output = assert_bash_exec(
            bash, '_comp__test_compgen split -- ""', want_output=True
        )
        assert output.strip() == ""

    def test_5_empty2(self, bash, functions):
        output = assert_bash_exec(
            bash, '_comp__test_compgen split -- " "', want_output=True
        )
        assert output.strip() == ""



import pytest


class TestCurl:
    @pytest.mark.complete("curl --h", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("curl -o f", cwd="shared/default/foo.d")
    def test_2(self, completion):
        assert completion == "oo"

    @pytest.mark.complete("curl -LRo f", cwd="shared/default/foo.d")
    def test_3(self, completion):
        assert completion == "oo"

    @pytest.mark.complete("curl --o f")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("curl --data @", cwd="shared/default/foo.d")
    def test_data_atfile(self, completion):
        assert "@foo" in completion
        assert "@-" in completion

    @pytest.mark.complete("curl --data @foo.", cwd="shared/default")
    def test_data_atfile_dir(self, completion):
        assert completion == "d/"
        assert not completion.endswith(" ")

    @pytest.mark.complete("curl --proto-default ", require_cmd=True)
    def test_proto_default(self, completion):
        assert completion

    @pytest.mark.complete("curl --dont-fail-in-unset-mode")
    def test_unknown_option(self, completion):
        # Just see that it does not error out
        pass

    @pytest.mark.complete("curl --data-bina", require_cmd=True)
    def test_help_all_option(self, completion):
        """
        The option used as a canary here is one that should be available
        in all curl versions. It should be only listed in `--help all` output
        for curl versions that have their help output split to multiple
        categories (i.e. ones that support `--help all` to get the complete
        list), as well as the basic `--help` output for earlier versions that
        do not have that.
        """
        assert completion



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+(VAR=|declare -f foo)")
class TestUnlocal:
    def test_1(self, bash):
        cmd = (
            "foo() { "
            "local VAR=inner; "
            "_comp_unlocal VAR; "
            "echo $VAR; "
            "}; "
            "VAR=outer; foo; "
        )
        res = assert_bash_exec(bash, cmd, want_output=True).strip()
        assert res == "outer"



import pytest

from conftest import assert_bash_exec


class TestXfreerdp:
    def _help(self, bash):
        return assert_bash_exec(
            bash, "xfreerdp --help 2>&1 || :", want_output=True
        )

    @pytest.fixture(scope="class")
    def help_success(self, bash):
        output = self._help(bash)
        # Example from our CentOS 7 container
        # [04:51:31:663] [238:238] [ERROR][com.freerdp.client.x11] - Failed to get pixmap info
        if not output or "ERROR" in output.strip().splitlines()[0]:
            pytest.skip("--help errored")

    @pytest.fixture(scope="class")
    def slash_syntax(self, bash, help_success):
        if "/help" not in self._help(bash):
            pytest.skip("Not slash syntax")

    @pytest.fixture(scope="class")
    def dash_syntax(self, bash):
        if "/help" in self._help(bash):
            pytest.skip("Not dash syntax")

    @pytest.mark.complete("xfreerdp /", require_cmd=True)
    def test_1(self, bash, completion, help_success, slash_syntax):
        assert completion

    @pytest.mark.complete("xfreerdp -", require_cmd=True)
    def test_2(self, completion, help_success):
        assert completion

    @pytest.mark.complete("xfreerdp +", require_cmd=True)
    def test_3(self, bash, completion, help_success, slash_syntax):
        assert completion

    @pytest.mark.complete(
        "xfreerdp /kbd:",
        require_cmd=True,
        skipif='test ! "$(xfreerdp /kbd-list 2>/dev/null)"',
    )
    def test_4(self, bash, completion, help_success, slash_syntax):
        assert completion

    @pytest.mark.complete("xfreerdp /help ", require_cmd=True)
    def test_5(self, completion, help_success):
        assert not completion

    @pytest.mark.complete("xfreerdp -k ", require_cmd=True)
    def test_6(self, bash, completion, help_success, dash_syntax):
        assert completion

    @pytest.mark.complete("xfreerdp --help ", require_cmd=True)
    def test_7(self, completion):
        assert not completion

    @pytest.mark.complete("xfreerdp ./")
    def test_rdp_files(self, completion):
        assert completion  # just dirs for now in the fixture, but that'll do



import pytest


class TestPv:
    @pytest.mark.complete("pv ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("pv -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("pv --pidfile ")
    def test_3(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(
    cmd="dnssec-keygen", pre_cmds=("PATH=$PATH:$PWD/dnssec-keygen",)
)
class TestDnssecKeygen:
    @pytest.mark.complete("dnssec-keygen -")
    def test_1(self, completion):
        assert completion
        assert not any(x.endswith(":") for x in completion)

    @pytest.mark.complete("dnssec-keygen -a ")
    def test_2(self, completion):
        assert completion
        assert any(x in completion for x in ("HMAC-MD5", "RSASHA1", "ED25519"))
        assert "|" not in completion
        assert not any(x.startswith("-") for x in completion)

    @pytest.mark.complete("dnssec-keygen -n ")
    def test_3(self, completion):
        assert completion
        assert "HOST" in completion
        assert "|" not in completion
        assert not any(x.startswith("-") for x in completion)

    @pytest.mark.complete("dnssec-keygen -f ")
    def test_4(self, completion):
        assert completion
        assert "|" not in completion
        assert not any(x.startswith("-") for x in completion)

    @pytest.mark.complete("dnssec-keygen ")
    def test_5(self, completion):
        assert not completion

    @pytest.mark.complete(
        "dnssec-keygen -a ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_6(self, completion):
        assert completion == sorted(
            "RSA RSAMD5 DSA RSASHA1 NSEC3RSASHA1 NSEC3DSA "
            "RSASHA256 RSASHA512 ECCGOST "
            "ECDSAP256SHA256 ECDSAP384SHA384 "
            "ED25519 ED448 DH "
            "HMAC-MD5 HMAC-SHA1 HMAC-SHA224 HMAC-SHA256 "
            "HMAC-SHA384 HMAC-SHA512".split()
        )

    @pytest.mark.complete(
        "dnssec-keygen -n ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_7(self, completion):
        assert completion == sorted("ZONE HOST ENTITY USER OTHER".split())

    @pytest.mark.complete(
        "dnssec-keygen -f ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_8(self, completion):
        assert completion == sorted("KSK REVOKE".split())

    @pytest.mark.complete(
        "dnssec-keygen -T ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_9(self, completion):
        assert completion == sorted("DNSKEY KEY".split())

    @pytest.mark.complete(
        "dnssec-keygen -t ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_10(self, completion):
        assert completion == sorted(
            "AUTHCONF NOAUTHCONF NOAUTH NOCONF".split()
        )

    @pytest.mark.complete(
        "dnssec-keygen -m ", env=dict(PATH="$PWD/dnssec-keygen:$PATH")
    )
    def test_11(self, completion):
        assert completion == sorted("usage trace record size mctx".split())



import pytest


class TestScrub:
    @pytest.mark.complete("scrub ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("scrub -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    # Not all scrub versions list available patterns in --help output
    @pytest.mark.complete(
        "scrub -p ",
        xfail="! (scrub --help 2>&1 || :) | command grep -q ^Available",
    )
    def test_3(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(ignore_env=r"^\+_comp_cmd_tshark__pr(ef|otocol)s=")
class TestTshark:
    @pytest.mark.complete("tshark -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("tshark -G ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("tshark -O foo,htt", require_cmd=True)
    def test_3(self, completion):
        # p: one completion only; http: e.g. http and http2
        assert completion == "p" or "foo,http" in completion

    @pytest.mark.complete("tshark -o tcp", require_cmd=True)
    def test_4(self, completion):
        assert "tcp.desegment_tcp_streams:" in completion

    @pytest.mark.complete("tshark -otcp", require_cmd=True)
    def test_5(self, completion):
        assert "-otcp.desegment_tcp_streams:" in completion

    @pytest.mark.complete("tshark -O http")
    def test_6(self, completion):
        """Test there are no URLs in completions."""
        assert not any("://" in x for x in completion)

    @pytest.mark.complete("tshark -r ")
    def test_input_files(self, completion):
        assert completion



import pytest


class TestSu:
    @pytest.mark.complete("su ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "su -", require_cmd=True, skipif="[[ $OSTYPE != *linux* ]]"
    )
    def test_2(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(cmd="ecryptfs-migrate-home")
class TestEcryptfsMigrateHome:
    @pytest.mark.complete("ecryptfs-migrate-home ", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ecryptfs-migrate-home -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestTruncate:
    @pytest.mark.complete("truncate ")
    def test_basic(self, completion):
        assert completion

    @pytest.mark.complete("truncate -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import pytest


class TestTox:
    @pytest.mark.complete("tox -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("tox -e ", cwd="tox")
    def test_2(self, completion):
        assert all(x in completion for x in "py37 ALL".split())

    @pytest.mark.complete("tox -e foo,", cwd="tox")
    def test_3(self, completion):
        assert all("foo," + x in completion for x in "py37 ALL".split())

    @pytest.mark.complete("tox -e foo -- ", cwd="tox")
    def test_default_after_dashdash(self, completion):
        assert "tox.ini" in completion



import pytest


class TestKill:
    @pytest.mark.complete("kill 1", xfail="! type ps &>/dev/null")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("kill -s ")
    def test_2(self, completion):
        assert all(x in completion for x in "HUP QUIT".split())

    @pytest.mark.complete("kill -")
    def test_3(self, completion):
        assert all("-%s" % x in completion for x in "l s ABRT USR1".split())

    @pytest.mark.complete("kill %", pre_cmds=("bash -c 'sleep 5' &",))
    def test_jobs(self, bash, completion):
        assert "bash" in completion



import pytest


class TestA2ps:
    @pytest.mark.complete("a2ps ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("a2ps -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import pytest

from conftest import TestUnitBase, assert_bash_exec


@pytest.mark.bashcomp(
    cmd=None,
    ignore_env=r"^\+declare -f __tester$",
)
class TestUnitQuote(TestUnitBase):
    def test_1(self, bash):
        assert_bash_exec(
            bash,
            '__tester() { local REPLY; _comp_quote "$1"; printf %s "$REPLY"; }',
        )
        output = assert_bash_exec(
            bash, '__tester "a b"', want_output=True, want_newline=False
        )
        assert output.strip() == "'a b'"

    def test_2(self, bash):
        output = assert_bash_exec(
            bash, '__tester "a  b"', want_output=True, want_newline=False
        )
        assert output.strip() == "'a  b'"

    def test_3(self, bash):
        output = assert_bash_exec(
            bash, '__tester "  a "', want_output=True, want_newline=False
        )
        assert output.strip() == "'  a '"

    def test_4(self, bash):
        output = assert_bash_exec(
            bash, "__tester \"a'b'c\"", want_output=True, want_newline=False
        )
        assert output.strip() == r"'a'\''b'\''c'"

    def test_5(self, bash):
        output = assert_bash_exec(
            bash, '__tester "a\'"', want_output=True, want_newline=False
        )
        assert output.strip() == r"'a'\'''"



import getpass

import pytest

from conftest import assert_complete


@pytest.mark.bashcomp(
    pre_cmds=(
        # Fake root command to get all users/groups completed at least for now
        "_comp_root_command=sudo",
    )
)
class TestChown:
    @pytest.mark.xfail(
        getpass.getuser() != "root", reason="Only root can chown to all users"
    )
    @pytest.mark.complete("chown ")
    def test_1(self, bash, completion, output_sort_uniq):
        users = output_sort_uniq("compgen -u")
        assert completion == users

    @pytest.mark.complete("chown foo: shared/default/")
    def test_2(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete("chown :foo shared/default/")
    def test_3(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    def test_4(self, bash, part_full_user):
        part, full = part_full_user
        completion = assert_complete(bash, "chown %s" % part)
        assert completion == full[len(part) :]
        assert completion.endswith(" ")

    def test_5(self, bash, part_full_user, part_full_group):
        _, user = part_full_user
        partgroup, fullgroup = part_full_group
        completion = assert_complete(bash, "chown %s:%s" % (user, partgroup))
        assert completion == fullgroup[len(partgroup) :]
        assert completion.output.endswith(" ")

    def test_6(self, bash, part_full_group):
        part, full = part_full_group
        completion = assert_complete(bash, "chown dot.user:%s" % part)
        assert completion == full[len(part) :]
        assert completion.output.endswith(" ")

    @pytest.mark.parametrize(
        "prefix",
        [
            r"funky\ user:",
            "funky.user:",
            r"funky\.user:",
            r"fu\ nky.user:",
            r"f\ o\ o\.\bar:",
            r"foo\_b\ a\.r\ :",
        ],
    )
    def test_7(self, bash, part_full_group, prefix):
        """Test preserving special chars in $prefix$partgroup<TAB>."""
        part, full = part_full_group
        completion = assert_complete(bash, "chown %s%s" % (prefix, part))
        assert completion == full[len(part) :]
        assert completion.output.endswith(" ")

    def test_8(self, bash, part_full_user, part_full_group):
        """Test giving up on degenerate cases instead of spewing junk."""
        _, user = part_full_user
        partgroup, _ = part_full_group
        for x in range(2, 5):
            completion = assert_complete(
                bash, "chown %s%s:%s" % (user, x * "\\", partgroup)
            )
            assert not completion

    def test_9(self, bash, part_full_group):
        """Test graceful fail on colon in user/group name."""
        part, _ = part_full_group
        completion = assert_complete(bash, "chown foo:bar:%s" % part)
        assert not completion



import os

import pytest

from conftest import in_container


class TestIfdown:
    @pytest.mark.xfail(
        os.environ.get("NETWORK") == "none",
        reason="There won't be any configured interfaces without network",
    )
    @pytest.mark.xfail(in_container(), reason="Probably fails in a container")
    @pytest.mark.complete("ifdown ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ifdown bash-completion ")
    def test_2(self, completion):
        assert not completion



import pytest


class TestHash:
    @pytest.mark.complete("hash ", require_cmd=True)
    def test_basic(self, completion):
        assert completion

    @pytest.mark.complete("hash -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import pytest


class TestPhing:
    @pytest.mark.complete("phing -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("phing -l ", require_cmd=True)
    def test_2(self, completion):
        assert not completion



import pytest

from conftest import assert_complete, create_dummy_filedirs


@pytest.mark.bashcomp(temp_cwd=True)
class TestEvince:
    def test_1(self, bash):
        files, dirs = create_dummy_filedirs(
            (
                ".cb7 .CB7 .cbr .CBR .cbt .CBT .cbz .CBZ .djv .DJV .djvu "
                ".DJVU .dvi .DVI .dvi.bz2 .dvi.BZ2 .DVI.bz2 .DVI.BZ2 .dvi.gz "
                ".dvi.GZ .DVI.gz .DVI.GZ .eps .EPS .eps.bz2 .eps.BZ2 .EPS.bz2 "
                ".EPS.BZ2 .eps.gz .eps.GZ .EPS.gz .EPS.GZ .oxps .OXPS .pdf "
                ".PDF .pdf.bz2 .pdf.BZ2 .PDF.bz2 .PDF.BZ2 .pdf.gz .pdf.GZ "
                ".PDF.gz .PDF.GZ .ps .PS .ps.bz2 .ps.BZ2 .PS.bz2 .PS.BZ2 "
                ".ps.gz .ps.GZ .PS.gz .PS.GZ .tif .TIF .tiff .TIFF .txt .TXT "
                ".xps .XPS"
            ).split(),
            "foo".split(),
        )

        completion = assert_complete(bash, "evince ")
        assert completion == [
            x
            for x in sorted(files + ["%s/" % d for d in dirs])
            if x.lower() != ".txt"
        ]

    @pytest.mark.complete("evince -", require_cmd=True)
    def test_2(self, completion):
        assert completion



# Based on work by Stephen Gildea, October 2010.

import pytest

from conftest import assert_bash_exec, bash_env_saved


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+declare -f fn$")
class TestUnitParseHelp:
    def test_1(self, bash):
        assert_bash_exec(bash, "fn() { echo; }")
        output = assert_bash_exec(bash, "_parse_help fn; (($? == 1))")
        assert not output

    def test_2(self, bash):
        assert_bash_exec(bash, "fn() { echo 'no dashes here'; }")
        output = assert_bash_exec(bash, "_parse_help fn; (($? == 1))")
        assert not output

    def test_3(self, bash):
        assert_bash_exec(bash, "fn() { echo 'internal-dash'; }")
        output = assert_bash_exec(bash, "_parse_help fn; (($? == 1))")
        assert not output

    def test_4(self, bash):
        assert_bash_exec(bash, "fn() { echo 'no -leading-dashes'; }")
        output = assert_bash_exec(bash, "_parse_help fn; (($? == 1))")
        assert not output

    def test_5(self, bash):
        assert_bash_exec(bash, "fn() { echo '-one dash'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "-one".split()

    def test_6(self, bash):
        assert_bash_exec(bash, "fn() { echo ' -space dash'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "-space".split()

    def test_7(self, bash):
        assert_bash_exec(bash, "fn() { echo '-one -two dashes'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "-one".split()

    def test_8(self, bash):
        assert_bash_exec(bash, "fn() { echo '-one,-t dashes'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "-one".split()

    def test_9(self, bash):
        assert_bash_exec(bash, "fn() { echo '-one dash-inside'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "-one".split()

    def test_10(self, bash):
        """Test value not included in completion."""
        assert_bash_exec(bash, "fn() { echo '--long-arg=value'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--long-arg=".split()

    def test_11(self, bash):
        """Test -value not seen as option."""
        assert_bash_exec(bash, "fn() { echo '--long-arg=-value'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--long-arg=".split()

    def test_12(self, bash):
        assert_bash_exec(bash, "fn() { echo '--long-arg=-value,--opt2=val'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--long-arg=".split()

    def test_13(self, bash):
        assert_bash_exec(bash, "fn() { echo '-m,--mirror'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--mirror".split()

    def test_14(self, bash):
        assert_bash_exec(bash, "fn() { echo '-T/--upload-file'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--upload-file".split()

    def test_15(self, bash):
        assert_bash_exec(bash, "fn() { echo '-T|--upload-file'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--upload-file".split()

    def test_16(self, bash):
        assert_bash_exec(bash, "fn() { echo '-f, -F, --foo'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_17(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo[=bar]'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_17_failglob(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo[=bar]'; }")
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("failglob", True)
            output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_17_nullglob(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo[=bar]'; }")
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("nullglob", True)
            output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_18(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo=<bar>'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo=".split()

    def test_19(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo={bar,quux}'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo=".split()

    def test_20(self, bash):
        assert_bash_exec(bash, "fn() { echo '--[no]foo'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo --nofoo".split()

    def test_21(self, bash):
        assert_bash_exec(bash, "fn() { echo '--[no-]bar[=quux]'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--bar --no-bar".split()

    def test_22(self, bash):
        assert_bash_exec(bash, "fn() { echo '--[no-]bar=quux'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--bar= --no-bar=".split()

    def test_23(self, bash):
        assert_bash_exec(bash, "fn() { echo '--[dont-]foo'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo --dont-foo".split()

    def test_24(self, bash):
        assert_bash_exec(bash, "fn() { echo '-[dont]x --[dont]yy'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--yy --dontyy".split()

    def test_25(self, bash):
        assert_bash_exec(bash, "fn() { echo '-f FOO, --foo=FOO'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo=".split()

    def test_26(self, bash):
        assert_bash_exec(bash, "fn() { echo '-f [FOO], --foo[=FOO]'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_27(self, bash):
        assert_bash_exec(bash, "fn() { echo '--foo.'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_27_middle_dot(self, bash):
        """We do not want to include the period at the end of the sentence but
        want to include dots connecting names."""
        assert_bash_exec(bash, "fn() { echo '--foo.bar.'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo.bar".split()

    def test_28(self, bash):
        assert_bash_exec(bash, "fn() { echo '-f or --foo'; }")
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--foo".split()

    def test_29(self, bash):
        """Test parsing from stdin."""
        output = assert_bash_exec(
            bash, "echo '-f or --foo' | _parse_help -", want_output=True
        )
        assert output.split() == "--foo".split()

    def test_30(self, bash):
        """More than two dashes should not be treated as options."""
        assert_bash_exec(
            bash, r"fn() { printf '%s\n' $'----\n---foo\n----- bar'; }"
        )
        output = assert_bash_exec(bash, "_parse_help fn; (($? == 1))")
        assert not output

    def test_31(self, bash):
        assert_bash_exec(
            bash,
            r"fn() { printf '%s\n' "
            r"'-F ERROR_FORMAT, --error-format ERROR_FORMAT'; }",
        )
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--error-format".split()

    def test_32(self, bash):
        assert_bash_exec(
            bash,
            r"fn() { printf '%s\n' "
            r"'-e CODE1,CODE2..  --exclude=CODE1,CODE2..'; }",
        )
        output = assert_bash_exec(bash, "_parse_help fn", want_output=True)
        assert output.split() == "--exclude=".split()

    def test_custom_helpopt1(self, bash):
        assert_bash_exec(bash, "fn() { [[ $1 == -h ]] && echo '-option'; }")
        output = assert_bash_exec(bash, "_parse_help fn -h", want_output=True)
        assert output.split() == "-option".split()

    def test_custom_helpopt2(self, bash):
        assert_bash_exec(bash, "fn() { [[ $1 == '-?' ]] && echo '-option'; }")
        output = assert_bash_exec(
            bash, "_parse_help fn '-?'", want_output=True
        )
        assert output.split() == "-option".split()

    def test_custom_helpopt2_failglob(self, bash):
        assert_bash_exec(bash, "fn() { [[ $1 == '-?' ]] && echo '-option'; }")
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("failglob", True)
            output = assert_bash_exec(
                bash, "_parse_help fn '-?'", want_output=True
            )
        assert output.split() == "-option".split()

    def test_custom_helpopt2_nullglob(self, bash):
        assert_bash_exec(bash, "fn() { [[ $1 == '-?' ]] && echo '-option'; }")
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("nullglob", True)
            output = assert_bash_exec(
                bash, "_parse_help fn '-?'", want_output=True
            )
        assert output.split() == "-option".split()



import sys

import pytest

from conftest import assert_bash_exec


class TestUmount:
    @pytest.fixture(scope="class")
    def dummy_mnt(self, request, bash):
        """
        umount completion from fstab can't be tested directly because it
        (correctly) uses absolute paths. So we create a custom completion which
        reads from a file in our text fixture instead.
        """
        if sys.platform != "linux":
            pytest.skip("Linux specific")
        assert_bash_exec(bash, "unset COMPREPLY cur; unset -f _mnt_completion")
        assert_bash_exec(
            bash,
            "_mnt_completion() { "
            "local cur prev;_comp_get_words cur prev; "
            '_comp_cmd_umount__linux_fstab "$prev" < mount/test-fstab; '
            "} && complete -F _mnt_completion _mnt",
        )
        request.addfinalizer(
            lambda: assert_bash_exec(
                bash, "complete -r _mnt; unset -f _mnt_completion"
            )
        )

    @pytest.mark.complete("umount ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("_mnt /mnt/nice-test-p")
    def test_mnt_basic(self, completion, dummy_mnt):
        assert completion == "ath"

    # Note in tests below that return only one result, that the result
    # is shell unescaped due to how assert_complete handles the
    # "one result on same line case".

    @pytest.mark.complete(r"_mnt /mnt/nice\ test-p")
    def test_mnt_space(self, completion, dummy_mnt):
        assert completion == r"ath"

    @pytest.mark.complete(r"_mnt /mnt/nice\$test-p")
    def test_mnt_dollar(self, completion, dummy_mnt):
        assert completion == "ath"

    @pytest.mark.complete(r"_mnt /mnt/nice\ test\\p")
    def test_mnt_backslash(self, completion, dummy_mnt):
        assert completion == "ath"

    @pytest.mark.complete(r"_mnt /mnt/nice\ ")
    def test_mnt_after_space(self, completion, dummy_mnt):
        assert completion == sorted(
            (r"/mnt/nice\ test\\path", r"/mnt/nice\ test-path")
        )

    @pytest.mark.complete(r"_mnt /mnt/nice\$")
    def test_mnt_at_dollar(self, completion, dummy_mnt):
        assert completion == "test-path"

    @pytest.mark.complete(r"_mnt /mnt/nice\'")
    def test_mnt_at_quote(self, completion, dummy_mnt):
        assert completion == "test-path"

    @pytest.mark.complete("_mnt /mnt/other")
    def test_mnt_other(self, completion, dummy_mnt):
        assert completion == r"\'test\ path"

    @pytest.mark.complete("_mnt -L Ubu")
    def test_mnt_label_space(self, completion, dummy_mnt):
        assert completion == r"ntu\ Karmic"

    @pytest.mark.complete("_mnt -L Deb")
    def test_mnt_label_quote(self, completion, dummy_mnt):
        assert completion == r"ian-it\'s\ awesome"

    @pytest.mark.skipif(sys.platform != "linux", reason="Linux specific")
    def test_linux_fstab_unescape(self, bash):
        assert_bash_exec(bash, r"var=one\'two\\040three\\")
        assert_bash_exec(bash, "_comp_cmd_umount__linux_fstab_unescape var")
        output = assert_bash_exec(
            bash, r'printf "%s\n" "$var"', want_output=True
        )
        assert output.strip() == "one'two three\\"
        assert_bash_exec(bash, "unset var")



import pytest


class TestGroupadd:
    @pytest.mark.complete("groupadd ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("groupadd -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


def is_int(s):
    try:
        int(s)
    except ValueError:
        return False
    else:
        return True


class TestPs:
    @pytest.mark.complete("ps -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ps --help ")
    def test_2(self, completion):
        assert completion == [
            "all",
            "list",
            "misc",
            "output",
            "simple",
            "threads",
        ]

    @pytest.mark.complete("ps --help all ")
    def test_3(self, completion):
        assert not completion

    @pytest.mark.complete("ps --version ")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("ps --pid ")
    def test_5(self, completion):
        assert completion
        assert all(map(is_int, completion))

    @pytest.mark.complete("ps --format ", require_cmd=True)
    def test_6(self, completion):
        assert completion
        assert all(map(lambda c: not c.startswith(("-", ",")), completion))

    @pytest.mark.complete("ps --format user,", require_cmd=True)
    def test_7(self, completion):
        assert completion
        assert all(map(lambda c: not c.startswith(("-", ",")), completion))



import os

import pytest

from conftest import assert_complete


class TestMake:
    @pytest.mark.complete("make -f Ma", cwd="make")
    def test_1(self, completion):
        assert completion == "kefile"

    @pytest.mark.complete("make .", cwd="make", require_cmd=True)
    def test_2(self, bash, completion):
        """Hidden targets."""
        assert completion == ".cache/ .test_passes".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make .cache/", cwd="make", require_cmd=True)
    def test_3(self, bash, completion):
        assert completion == ".cache/1 .cache/2".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make ", cwd="shared/empty_dir")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("make -j ")
    def test_5(self, completion):
        assert completion

    @pytest.mark.complete("make ", cwd="make", require_cmd=True)
    def test_6(self, bash, completion):
        assert completion == "all clean extra_makefile install sample".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make .cache/.", cwd="make", require_cmd=True)
    def test_7(self, bash, completion):
        assert completion == ".cache/.1 .cache/.2".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make -C make ", require_cmd=True)
    def test_8(self, bash, completion):
        assert completion == "all clean extra_makefile install sample".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make -nC make ", require_cmd=True)
    def test_8n(self, bash, completion):
        assert completion == "all clean extra_makefile install sample".split()
        os.remove(f"{bash.cwd}/make/extra_makefile")

    @pytest.mark.complete("make -", require_cmd=True)
    def test_9(self, completion):
        assert completion


@pytest.mark.bashcomp(require_cmd=True, cwd="make/test2")
class TestMake2:
    def test_github_issue_544_1(self, bash):
        completion = assert_complete(bash, "make ab")
        assert completion == "c/xyz"

    def test_github_issue_544_2(self, bash):
        completion = assert_complete(bash, "make 1")
        assert completion == "23/"

    def test_github_issue_544_3(self, bash):
        completion = assert_complete(bash, "make 123/")
        assert completion == ["123/xaa", "123/xbb"]

    def test_github_issue_544_4(self, bash):
        completion = assert_complete(bash, "make 123/xa")
        assert completion == "a"

    def test_subdir_1(self, bash):
        completion = assert_complete(bash, "make sub1")
        assert completion == "test/bar/"

    def test_subdir_2(self, bash):
        completion = assert_complete(bash, "make sub2")
        assert completion == "test/bar/alpha"

    def test_subdir_3(self, bash):
        completion = assert_complete(bash, "make sub3")
        assert completion == "test/"

    def test_subdir_4(self, bash):
        completion = assert_complete(bash, "make sub4")
        assert completion == "sub4test/bar/ sub4test2/foo/gamma".split()



import pytest


class TestPerlcritic:
    @pytest.mark.complete("perlcritic ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("perlcritic --", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("perlcritic --theme ", require_cmd=True)
    def test_3(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(cmd="apt-cache")
class TestAptCache:
    @pytest.mark.complete("apt-cache ")
    def test_1(self, completion):
        assert "search" in completion

    @pytest.mark.complete("apt-cache showsrc [", require_cmd=True)
    def test_2(self, completion):
        # Doesn't actually fail on grep errors, but takes a long time.
        assert not completion

    @pytest.mark.complete("apt-cache ", trail=" add foo")
    def test_special_at_point(self, completion):
        assert not completion



import pytest


class TestCat:
    @pytest.mark.complete("cat ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("cat -", require_longopt=True)
    def test_options(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(cmd="gnome-screenshot")
class TestGnomeScreenshot:
    @pytest.mark.complete("gnome-screenshot --help", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest

from conftest import assert_bash_exec


class TestCancel:
    @pytest.fixture(scope="class")
    def added_job(self, request, bash):
        try:
            got = (
                assert_bash_exec(
                    bash, "lp -H hold shared/default/foo", want_output=True
                )
                .strip()
                .split()
            )
        except AssertionError:
            pytest.skip("Could not add test print job")
            return
        if len(got) > 3:
            request.addfinalizer(
                lambda: assert_bash_exec(bash, "cancel %s" % got[3])
            )

    @pytest.mark.complete("cancel ")
    def test_1(self, bash, completion, added_job):
        got = (
            assert_bash_exec(
                bash, "lpstat | awk '{print $1}'", want_output=True
            )
            .strip()
            .split()
        )
        assert completion == sorted(got)



import pytest


@pytest.mark.bashcomp(pre_cmds=("HOME=$PWD/cvs",))
class TestCvs:
    @pytest.mark.complete("cvs ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("cvs -d ")
    def test_2(self, completion):
        assert [x for x in completion if ":pserver:" in x]

    @pytest.mark.complete("cvs diff foo/", cwd="cvs")
    def test_3(self, completion):
        assert completion == "bar"

    @pytest.mark.complete("cvs -", require_cmd=True)
    def test_4(self, completion):
        assert completion

    @pytest.mark.complete("cvs update -AdP foo/", cwd="cvs")
    def test_5(self, completion):
        assert completion == "bar"

    @pytest.mark.complete("cvs log -v foo/", cwd="cvs")
    def test_6(self, completion):
        assert completion == "bar"

    @pytest.mark.complete("cvs diff foo/", cwd="cvs")
    def test_7(self, completion):
        assert completion == "bar"

    @pytest.mark.complete("cvs status -v foo/", cwd="cvs")
    def test_8(self, completion):
        assert completion == "bar"

    @pytest.mark.complete("cvs status foo/", cwd="cvs")
    def test_9(self, completion):
        assert completion == "bar"



import pytest


class TestSqlite3:
    @pytest.mark.complete("sqlite3 ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("sqlite3 -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("sqlite3 -scratch foo ", require_cmd=True)
    def test_3(self, completion):
        assert not completion



import pytest


class TestXgamma:
    @pytest.mark.complete("xgamma -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("xgamma -gamm", require_cmd=True)
    def test_2(self, completion):
        assert completion == "a"
        assert completion.endswith(" ")



import pytest


class TestMunindoc:
    # Assume at least munin* available
    # require_cmd is not strictly correct here, but...
    @pytest.mark.complete("munindoc m", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest


class TestFind:
    @pytest.mark.complete("find ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("find -fstype ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("find -")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("find -wholename ", cwd="shared/default")
    def test_4(self, completion):
        assert completion == ["bar", "bar bar.d/", "foo", "foo.d/"]

    @pytest.mark.complete("find -uid ")
    def test_5(self, completion):
        assert not [x for x in completion if not x.isdigit()]

    @pytest.mark.complete("find -gid ")
    def test_6(self, completion):
        assert not [x for x in completion if not x.isdigit()]

    @pytest.mark.complete("find -exec shared/bin/ar")
    def test_exec(self, completion):
        assert completion == "p"

    # sh +: something that produces completions also when command is not
    #       available, and the chosen completion is not one of find's
    @pytest.mark.complete("find /some/where -exec sh +")
    def test_exec_args(self, completion):
        assert "+o" in completion



import pytest

from conftest import assert_bash_exec, assert_complete


@pytest.mark.bashcomp(ignore_env=r"^\+declare -f fn$")
class TestFunction:
    @pytest.mark.complete("function _parse_")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("function non_existent_function ")
    def test_2(self, completion):
        assert completion == "()"

    def test_3(self, bash):
        assert_bash_exec(bash, "fn() { echo; }")
        completion = assert_complete(bash, "function fn ")
        assert completion == "() { ^J    echo^J}"



import pytest


class TestGetconf:
    @pytest.mark.complete("getconf P", skipif="! getconf -a &>/dev/null")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("getconf -")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("getconf -a ")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete(
        "getconf -v ", xfail="! getconf -a 2>&1 | command grep -q ^POSIX_V"
    )
    def test_4(self, completion):
        assert completion

    @pytest.mark.complete("getconf PATH_MAX ")
    def test_5(self, completion):
        assert completion



import pytest


class TestLsscsi:
    @pytest.mark.complete("lsscsi ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("lsscsi -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestWget:
    @pytest.mark.complete("wget ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("wget --s", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


class TestWtf:
    # TODO: actually requires an acronym db, not the cmd
    @pytest.mark.complete("wtf A", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest


class TestPdftoppm:
    @pytest.mark.complete("pdftoppm ")
    def test_files(self, completion):
        assert completion

    @pytest.mark.complete("pdftoppm -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import pytest


class TestLz4:
    @pytest.mark.complete("lz4 ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lz4 ~")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("lz4 -", require_cmd=True)
    def test_3(self, completion):
        assert completion



import os.path
from tempfile import mkstemp

import pytest

from conftest import assert_complete, is_bash_type


@pytest.mark.bashcomp(cmd="slapt-get")
class TestSlaptGet:
    @pytest.fixture(scope="class")
    def slapt_getrc(self, request, bash):
        fd, fname = mkstemp(prefix="slapt-getrc.", text=True)
        request.addfinalizer(lambda: os.remove(fname))
        with os.fdopen(fd, "w") as f:
            print(
                "WORKINGDIR=%s/"
                % os.path.join(bash.cwd, *"slackware var slapt-get".split()),
                file=f,
            )
            print("SOURCE=file:///home/", file=f)
        return fname

    @pytest.mark.complete("slapt-get -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("slapt-get --up", require_cmd=True)
    def test_2(self, completion):
        assert completion == "--update --upgrade".split()

    @pytest.mark.complete("slapt-get -c non-existent-file --install ")
    def test_3(self, completion):
        assert not completion

    def test_install(self, bash, slapt_getrc):
        if not is_bash_type(bash, "slapt-get"):
            pytest.skip("slapt-get not found")
        completion = assert_complete(
            bash, "slapt-get -c %s --install " % slapt_getrc
        )
        assert completion == sorted(
            "abc-4-i686-1 ran-1.2-noarch-1 qwe-2.1-i486-1".split()
        )



import pytest


class TestJavadoc:
    @pytest.mark.complete("javadoc ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("javadoc -linkoffline shared/default/")
    def test_2(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]

    @pytest.mark.complete(
        "javadoc -nodeprecated -linkoffline foo shared/default/"
    )
    def test_3(self, completion):
        assert completion == ["bar bar.d/", "foo.d/"]



import pytest


class TestPyston:
    @pytest.mark.complete("pyston ")
    def test_basic(self, completion):
        assert completion

    @pytest.mark.complete("pyston -", require_cmd=True)
    def test_options(self, completion):
        assert completion

    @pytest.mark.complete(
        "pyston -b",
        require_cmd=True,
        skipif="! pyston -h | command grep -qwF -- -bb",
    )
    def test_bb(self, completion):
        assert "-bb" in completion



import pytest


@pytest.mark.bashcomp(cmd="xdg-settings")
class TestXdgSettings:
    @pytest.mark.complete("xdg-settings ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("xdg-settings --", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("xdg-settings get ", require_cmd=True)
    def test_3(self, completion):
        assert completion



import pytest


class TestPtx:
    @pytest.mark.complete("ptx ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ptx -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import re

import pytest

from conftest import assert_bash_exec, assert_complete, bash_env_saved


@pytest.mark.bashcomp(cmd=None)
class TestUtilCompgen:
    @pytest.fixture
    def functions(self, bash):
        assert_bash_exec(
            bash,
            "_comp__test_dump() { ((${#arr[@]})) && printf '<%s>' \"${arr[@]}\"; echo; }",
        )
        assert_bash_exec(
            bash,
            '_comp__test_compgen() { local -a arr=(00); _comp_compgen -v arr "$@"; _comp__test_dump; }',
        )
        assert_bash_exec(
            bash,
            '_comp__test_words() { local -a input=("${@:1:$#-1}"); _comp__test_compgen -c "${@:$#}" -- -W \'${input[@]+"${input[@]}"}\'; }',
        )
        assert_bash_exec(
            bash,
            '_comp__test_words_ifs() { local input=$2; _comp__test_compgen -F "$1" -c "${@:$#}" -- -W \'$input\'; }',
        )

        assert_bash_exec(
            bash,
            '_comp_cmd_fc() { _comp_compgen -c "$(_get_cword)" -C _filedir filedir; }; '
            "complete -F _comp_cmd_fc fc; "
            "complete -F _comp_cmd_fc -o filenames fc2",
        )
        assert_bash_exec(
            bash,
            '_comp_cmd_fcd() { _comp_compgen -c "$(_get_cword)" -C _filedir filedir -d; }; '
            "complete -F _comp_cmd_fcd fcd",
        )

        # test_8_option_U
        assert_bash_exec(
            bash,
            "_comp_compgen_gen8() { local -a arr=(x y z); _comp_compgen -U arr -- -W '\"${arr[@]}\"'; }",
        )

        # test_9_inherit_a
        assert_bash_exec(
            bash,
            '_comp_compgen_gen9sub() { local -a gen=(00); _comp_compgen -v gen -- -W 11; _comp_compgen_set "${gen[@]}"; }; '
            "_comp_compgen_gen9() { _comp_compgen_gen9sub; _comp_compgen -a gen9sub; }",
        )

    def test_1_basic(self, bash, functions):
        output = assert_bash_exec(
            bash, "_comp__test_words 12 34 56 ''", want_output=True
        )
        assert output.strip() == "<12><34><56>"

    def test_2_space(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_words $'a b' $'c d\\t' '  e  ' $'\\tf\\t' ''",
            want_output=True,
        )
        assert output.strip() == "<a b><c d\t><  e  ><\tf\t>"

    def test_2_IFS(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable("IFS", "34")
            output = assert_bash_exec(
                bash, "_comp__test_words 12 34 56 ''", want_output=True
            )
            assert output.strip() == "<12><34><56>"

    def test_3_glob(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_words '*' '[a-z]*' '[a][b][c]' ''",
            want_output=True,
        )
        assert output.strip() == "<*><[a-z]*><[a][b][c]>"

    def test_3_failglob(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("failglob", True)
            output = assert_bash_exec(
                bash,
                "_comp__test_words '*' '[a-z]*' '[a][b][c]' ''",
                want_output=True,
            )
            assert output.strip() == "<*><[a-z]*><[a][b][c]>"

    def test_3_nullglob(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("nullglob", True)
            output = assert_bash_exec(
                bash,
                "_comp__test_words '*' '[a-z]*' '[a][b][c]' ''",
                want_output=True,
            )
            assert output.strip() == "<*><[a-z]*><[a][b][c]>"

    def test_4_empty(self, bash, functions):
        output = assert_bash_exec(
            bash, "_comp__test_words ''", want_output=True
        )
        assert output.strip() == ""

    def test_5_option_F(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_words_ifs '25' ' 123 456 555 ' ''",
            want_output=True,
        )
        assert output.strip() == "< 1><3 4><6 >< >"

    def test_6_option_C_1(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_compgen -c a -C _filedir filedir",
            want_output=True,
        )
        set1 = set(re.findall(r"<[^<>]*>", output.strip()))
        assert set1 == {"<a b>", "<a$b>", "<a&b>", "<a'b>", "<ab>", "<aé>"}

    def test_6_option_C_2(self, bash, functions):
        output = assert_bash_exec(
            bash,
            "_comp__test_compgen -c b -C _filedir -- -d",
            want_output=True,
        )
        assert output.strip() == "<brackets>"

    @pytest.mark.parametrize("funcname", "fc fc2".split())
    def test_6_option_C_3(self, bash, functions, funcname):
        completion = assert_complete(bash, "%s _filedir ab/" % funcname)
        assert completion == "e"

    @pytest.mark.complete(r"fcd a\ ")
    def test_6_option_C_4(self, functions, completion):
        # Note: we are not in the original directory that "b" exists, so Bash
        # will not suffix a slash to the directory name.
        assert completion == "b"

    def test_7_icmd(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable(
                "BASH_COMPLETION_USER_DIR", "$PWD/_comp_compgen", quote=False
            )

            completions = assert_complete(bash, "compgen-cmd1 '")
            assert completions == ["012", "123", "234", "5abc", "6def", "7ghi"]

    def test_7_xcmd(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable(
                "BASH_COMPLETION_USER_DIR", "$PWD/_comp_compgen", quote=False
            )

            completions = assert_complete(bash, "compgen-cmd2 '")
            assert completions == ["012", "123", "234", "5foo", "6bar", "7baz"]

    def test_8_option_U(self, bash, functions):
        output = assert_bash_exec(
            bash, "_comp__test_compgen gen8", want_output=True
        )
        assert output.strip() == "<x><y><z>"

    def test_9_inherit_a(self, bash, functions):
        output = assert_bash_exec(
            bash, "_comp__test_compgen gen9", want_output=True
        )
        assert output.strip() == "<11><11>"



import pytest


class TestLsof:
    @pytest.mark.complete("lsof ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("lsof -")
    def test_2(self, completion):
        assert completion



import pexpect  # type: ignore[import]
import pytest

from conftest import PS1, TestUnitBase, assert_bash_exec


@pytest.mark.bashcomp(
    cmd=None,
    ignore_env=r"^[+-](COMP_(WORDS|CWORD|LINE|POINT)|_comp_cmd_scp__path_esc)=",
)
class TestUnitGetCword(TestUnitBase):
    def _test(self, *args, **kwargs):
        return self._test_unit("_get_cword %s; echo", *args, **kwargs)

    def test_1(self, bash):
        assert_bash_exec(
            bash,
            "COMP_WORDS=() COMP_CWORD= COMP_LINE= COMP_POINT= "
            "_get_cword >/dev/null",
        )

    def test_2(self, bash):
        """a b| should return b"""
        output = self._test(bash, "(a b)", 1, "a b", 3)
        assert output == "b"

    def test_3(self, bash):
        """a | should return nothing"""
        output = self._test(bash, "(a)", 1, "a ", 2)
        assert not output

    def test_4(self, bash):
        """a b | should return nothing"""
        output = self._test(bash, "(a b '')", 2, "a b ", 4)
        assert not output

    def test_5(self, bash):
        """a b | with WORDBREAKS -= : should return nothing"""
        output = self._test(bash, "(a b '')", 2, "a b ", 4, arg=":")
        assert not output

    def test_6(self, bash):
        """a b|c should return b"""
        output = self._test(bash, "(a bc)", 1, "a bc", 3)
        assert output == "b"

    def test_7(self, bash):
        r"""a b\ c| should return b\ c"""
        output = self._test(bash, r"(a 'b\ c')", 1, r"a b\ c", 6)
        assert output == r"b\ c"

    def test_8(self, bash):
        r"""a b\| c should return b\ """  # fmt: skip
        output = self._test(bash, r"(a 'b\ c')", 1, r"a b\ c", 4)
        assert output == "b\\"

    def test_9(self, bash):
        r"""a "b\| should return "b\ """  # fmt: skip
        output = self._test(bash, "(a '\"b\\')", 1, r"a \"b\\", 5)
        assert output == '"b\\'

    def test_10(self, bash):
        r"""a 'b c| should return 'b c"""
        output = self._test(bash, '(a "\'b c")', 1, "a 'b c", 6)
        assert output == "'b c"

    def test_11(self, bash):
        r"""a "b c| should return "b c"""
        output = self._test(bash, "(a '\"b c')", 1, 'a "b c', 6)
        assert output == '"b c'

    def test_12(self, bash):
        """a b:c| with WORDBREAKS += : should return c"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b : c)", 3, "a b:c", 5)
        assert output == "c"

    def test_13(self, bash):
        """a b:c| with WORDBREAKS -= : should return b:c"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b : c)", 3, "a b:c", 5, arg=":")
        assert output == "b:c"

    def test_14(self, bash):
        """a b c:| with WORDBREAKS -= : should return c:"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b c :)", 3, "a b c:", 6, arg=":")
        assert output == "c:"

    def test_15(self, bash):
        """a :| with WORDBREAKS -= : should return :"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a :)", 1, "a :", 3, arg=":")
        assert output == ":"

    def test_16(self, bash):
        """a b::| with WORDBREAKS -= : should return b::"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b::)", 1, "a b::", 5, arg=":")
        assert output == "b::"

    def test_17(self, bash):
        """
        a -n| should return -n

        This test makes sure `_get_cword' doesn't use `echo' to return its
        value, because -n might be interpreted by `echo' and thus would not
        be returned.
        """
        output = self._test(bash, "(a -n)", 1, "a -n", 4)
        assert output == "-n"

    def test_18(self, bash):
        """a b>c| should return c"""
        output = self._test(bash, r"(a b \> c)", 3, "a b>c", 5)
        assert output == "c"

    def test_19(self, bash):
        """a b=c| should return c"""
        output = self._test(bash, "(a b = c)", 3, "a b=c", 5)
        assert output == "c"

    def test_20(self, bash):
        """a *| should return *"""
        output = self._test(bash, r"(a \*)", 1, "a *", 4)
        assert output == "*"

    def test_21(self, bash):
        """a $(b c| should return $(b c"""
        output = self._test(bash, r"(a '$(b c')", 1, "a $(b c", 7)
        assert output == "$(b c"

    def test_22(self, bash):
        r"""a $(b c\ d| should return $(b c\ d"""
        output = self._test(bash, r"(a '$(b c\ d')", 1, r"a $(b c\ d", 10)
        assert output == r"$(b c\ d"

    def test_23(self, bash):
        """a 'b&c| should return 'b&c"""
        output = self._test(bash, '(a "\'b&c")', 1, "a 'b&c", 6)
        assert output == "'b&c"

    @pytest.mark.xfail(reason="TODO: non-ASCII issues with test suite?")
    def test_24(self, bash):
        """Index shouldn't drop below 0"""
        bash.send("scp ääää§ se\t\r\n")
        got = bash.expect_exact(
            [
                "index: substring expression < 0",
                PS1,
                pexpect.EOF,
                pexpect.TIMEOUT,
            ]
        )
        assert got == 1

    def test_25(self, bash):
        """
        a b c:| with trailing whitespace after the caret (no more words) and
        with WORDBREAKS -= : should return c:
        """
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b c :)", 3, "a b c: ", 6, arg=":")
        assert output == "c:"



import pytest


class TestCcache:
    @pytest.mark.complete("ccache -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ccache --clea", require_cmd=True)
    def test_2(self, completion):
        assert all(x in completion for x in "--cleanup --clear".split())

    @pytest.mark.complete("ccache stt")
    def test_3(self, completion):
        assert completion == "y" or "stty" in completion

    @pytest.mark.complete("ccache --zero-stats stt")
    def test_4(self, completion):
        assert completion == "y" or "stty" in completion

    @pytest.mark.complete("ccache --hel", require_cmd=True)
    def test_5(self, completion):
        assert completion == "p" or "--help" in completion

    @pytest.mark.complete("ccache --zero-stats sh +")
    def test_6(self, completion):
        assert "+x" in completion



import pytest


class TestXmlwf:
    @pytest.mark.complete("xmlwf ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("xmlwf -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("xmlwf -sa ")
    def test_no_arg_to_a(self, completion):
        assert not completion



import pytest


@pytest.mark.bashcomp(pre_cmds=("PATH=/usr/lib/mailman/bin:$PATH",))
class TestChangePw:
    @pytest.mark.complete("change_pw -")
    def test_1(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(cmd="secret-tool")
class TestSecretTool:
    @pytest.mark.complete("secret-tool ", require_cmd=True)
    def test_modes(self, completion):
        assert "store" in completion

    @pytest.mark.complete("secret-tool search ")
    def test_no_complete(self, completion):
        assert completion == "--all --unlock".split()



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+declare -f func[12]$")
class TestUnitDeprecateFunc:
    def test_1(self, bash):
        assert_bash_exec(
            bash,
            'func1() { echo "func1($*)"; }; '
            "_comp_deprecate_func 2.12 func2 func1",
        )
        output = assert_bash_exec(bash, "func2 1 2 3", want_output=True)
        assert output.strip() == "func1(1 2 3)"



import pytest


class TestMount:
    @pytest.mark.complete("mount ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("mount -t ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("mount /dev/sda1 def", cwd="shared")
    def test_3(self, completion):
        assert completion == "ault/"
        assert not completion.endswith(" ")

    @pytest.mark.complete(
        "mount mocksrv:/", env=dict(PATH="$PWD/mount/bin:$PATH")
    )
    def test_4(self, completion):
        assert completion == "/second/path /test/path /test/path2".split()



import pytest


class TestBzip2:
    @pytest.mark.complete("bzip2 ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("bzip2 ~")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("bzip2 -")
    def test_3(self, completion):
        assert completion



import pytest


class TestHtpasswd:
    @pytest.mark.complete("htpasswd ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("htpasswd -n htpasswd/ht")
    def test_2(self, completion):
        assert not completion

    @pytest.mark.complete("htpasswd ", cwd="htpasswd")
    def test_3(self, completion):
        assert completion == "htpasswd"

    @pytest.mark.complete("htpasswd -D htpasswd ", cwd="htpasswd")
    def test_4(self, completion):
        assert completion == "foo quux".split()

    @pytest.mark.complete("htpasswd -", require_cmd=True)
    def test_5(self, completion):
        assert completion



import pytest


class TestXmms:
    @pytest.mark.complete("xmms --", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("xmms --non-existent-option=--")
    def test_2(self, completion):
        assert not completion



import pytest

from conftest import TestUnitBase, assert_bash_exec


@pytest.mark.bashcomp(
    cmd=None,
    ignore_env=r"^(\+(words|cword|cur|prev)|[+-]COMP_(WORDS|CWORD|LINE|POINT))=",
)
class TestUnitGetCompWords(TestUnitBase):
    def _test(self, bash, *args, **kwargs):
        assert_bash_exec(bash, "unset cur prev")
        output = self._test_unit(
            '_comp_get_words %s cur prev; echo "$cur,${prev-}"',
            bash,
            *args,
            **kwargs,
        )
        return output.strip()

    def test_1(self, bash):
        assert_bash_exec(
            bash,
            "COMP_WORDS=() COMP_CWORD= COMP_POINT= COMP_LINE= "
            "_comp_get_words cur >/dev/null",
        )

    def test_2(self, bash):
        """a b|"""
        output = self._test(bash, "(a b)", 1, "a b", 3)
        assert output == "b,a"

    def test_3(self, bash):
        """a |"""
        output = self._test(bash, "(a)", 1, "a ", 2)
        assert output == ",a"

    def test_4(self, bash):
        """|a"""
        output = self._test(bash, "(a)", 0, "a", 0)
        assert output == ","

    def test_5(self, bash):
        """|a """  # fmt: skip
        output = self._test(bash, "(a)", 0, "a ", 0)
        assert output == ","

    def test_6(self, bash):
        """ | a """  # fmt: skip
        output = self._test(bash, "(a)", 0, "  a ", 1)
        assert output.strip() == ","

    def test_7(self, bash):
        """a b |"""
        output = self._test(bash, "(a b '')", 2, "a b ", 4)
        assert output == ",b"

    def test_8(self, bash):
        """a b | with WORDBREAKS -= :"""
        output = self._test(bash, "(a b '')", 2, "a b ", 4, arg="-n :")
        assert output == ",b"

    def test_9(self, bash):
        """a b|c"""
        output = self._test(bash, "(a bc)", 1, "a bc", 3)
        assert output == "b,a"

    def test_10(self, bash):
        """a | b"""
        output = self._test(bash, "(a b)", 1, "a  b", 2)
        assert output == ",a"

    def test_11(self, bash):
        r"""a b\ c|"""
        output = self._test(bash, r"(a 'b\ c')", 1, r"a b\ c", 6)
        assert output == r"b\ c,a"

    def test_12(self, bash):
        r"""a\ b a\ b|"""
        output = self._test(bash, r"('a\ b' 'a\ b')", 1, r"a\ b a\ b", 9)
        assert output == r"a\ b,a\ b"

    def test_13(self, bash):
        r"""a b\| c"""
        output = self._test(bash, r"(a 'b\ c')", 1, r"a b\ c", 4)
        assert output == r"b\,a"

    def test_14(self, bash):
        r"""a "b\|"""
        output = self._test(bash, "(a '\"b')", 1, 'a "b\\', 5)
        assert output == r'"b\,a'

    def test_15(self, bash):
        """a 'b c|"""
        output = self._test(bash, '(a "\'b c")', 1, "a 'b c", 6)
        assert output == "'b c,a"

    def test_16(self, bash):
        """a "b c|"""
        output = self._test(bash, r'(a "\"b c")', 1, 'a "b c', 6)
        assert output == '"b c,a'

    def test_17(self, bash):
        """a b:c| with WORDBREAKS += :"""
        assert_bash_exec(bash, "add_comp_wordbreak_char :")
        output = self._test(bash, "(a b : c)", 3, "a b:c", 5)
        assert output == "c,:"

    def test_18(self, bash):
        """a b:c| with WORDBREAKS -= :"""
        output = self._test(bash, "(a b : c)", 3, "a b:c", 5, arg="-n :")
        assert output == "b:c,a"

    def test_19(self, bash):
        """a b c:| with WORDBREAKS -= :"""
        output = self._test(bash, "(a b c :)", 3, "a b c:", 6, arg="-n :")
        assert output == "c:,b"

    def test_20(self, bash):
        r"""a b:c | with WORDBREAKS -= :"""
        output = self._test(bash, "(a b : c '')", 4, "a b:c ", 6, arg="-n :")
        assert output == ",b:c"

    def test_21(self, bash):
        """a :| with WORDBREAKS -= :"""
        output = self._test(bash, "(a :)", 1, "a :", 3, arg="-n :")
        assert output == ":,a"

    def test_22(self, bash):
        """a b::| with WORDBREAKS -= :"""
        output = self._test(bash, "(a b ::)", 2, "a b::", 5, arg="-n :")
        assert output == "b::,a"

    def test_23(self, bash):
        """a -n|

        This test makes sure `_comp_get_words' doesn't use `echo' to
        return its value, because -n might be interpreted by `echo'
        and thus would not be returned.
        """
        output = self._test(bash, "(a -n)", 1, "a -n", 4)
        assert output == "-n,a"

    def test_24(self, bash):
        """a b>c|"""
        output = self._test(bash, r"(a b \> c)", 3, "a b>c", 5)
        assert output.startswith("c,")

    def test_25(self, bash):
        """a b=c|"""
        output = self._test(bash, "(a b = c)", 3, "a b=c", 5)
        assert output.startswith("c,")

    def test_26(self, bash):
        """a *|"""
        output = self._test(bash, r"(a \*)", 1, "a *", 4)
        assert output == "*,a"

    def test_27(self, bash):
        """a $(b c|"""
        output = self._test(bash, "(a '$(b c')", 1, "a $(b c", 7)
        assert output == "$(b c,a"

    def test_28(self, bash):
        r"""a $(b c\ d|"""
        output = self._test(bash, r"(a '$(b c\ d')", 1, r"a $(b c\ d", 10)
        assert output == r"$(b c\ d,a"

    def test_29(self, bash):
        """a 'b&c|"""
        output = self._test(bash, '(a "\'b&c")', 1, "a 'b&c", 6)
        assert output == "'b&c,a"

    def test_30(self, bash):
        """a b| to all vars"""
        assert_bash_exec(bash, "unset words cword cur prev")
        output = self._test_unit(
            "_comp_get_words words cword cur prev%s; "
            'echo "${words[@]}",$cword,$cur,$prev',
            bash,
            "(a b)",
            1,
            "a b",
            3,
        )
        assert output == "a b,1,b,a"

    def test_31(self, bash):
        """a b| to alternate vars"""
        assert_bash_exec(bash, "unset words2 cword2 cur2 prev2")
        output = self._test_unit(
            "_comp_get_words -w words2 -i cword2 -c cur2 -p prev2%s; "
            'echo $cur2,$prev2,"${words2[@]}",$cword2',
            bash,
            "(a b)",
            1,
            "a b",
            3,
        )
        assert output == "b,a,a b,1"
        assert_bash_exec(bash, "unset words2 cword2 cur2 prev2")

    def test_32(self, bash):
        """a b : c| with wordbreaks -= :"""
        assert_bash_exec(bash, "unset words")
        output = self._test_unit(
            '_comp_get_words -n : words%s; echo "${words[@]}"',
            bash,
            "(a b : c)",
            3,
            "a b : c",
            7,
        )
        assert output == "a b : c"

    def test_33(self, bash):
        """a b: c| with wordbreaks -= :"""
        assert_bash_exec(bash, "unset words")
        output = self._test_unit(
            '_comp_get_words -n : words%s; echo "${words[@]}"',
            bash,
            "(a b : c)",
            3,
            "a b: c",
            6,
        )
        assert output == "a b: c"

    def test_34(self, bash):
        """a b :c| with wordbreaks -= :"""
        assert_bash_exec(bash, "unset words")
        output = self._test_unit(
            '_comp_get_words -n : words%s; echo "${words[@]}"',
            bash,
            "(a b : c)",
            3,
            "a b :c",
            6,
        )
        assert output == "a b :c"

    def test_35(self, bash):
        r"""a b\ :c| with wordbreaks -= :"""
        assert_bash_exec(bash, "unset words")
        output = self._test_unit(
            '_comp_get_words -n : words%s; echo "${words[@]}"',
            bash,
            "(a 'b ' : c)",
            3,
            r"a b\ :c",
            7,
        )
        assert output == "a b  :c"

    def test_unknown_arg_error(self, bash):
        with pytest.raises(AssertionError) as ex:
            _ = assert_bash_exec(
                bash, "_comp_get_words dummy", want_output=True
            )
        ex.match("dummy.* unknown argument")



import pytest


class TestVi:
    @pytest.mark.complete("vi ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("vi shared/ld.so.conf.d/")
    def test_2(self, completion):
        assert completion == "foo.txt libfoo.conf".split()



import pytest

from conftest import assert_complete


class TestSet:
    @pytest.mark.parametrize("dash", ["", "-", "--"])
    def test_basic(self, bash, dash):
        completion = assert_complete(bash, f"set {dash} ")
        assert completion

    @pytest.mark.parametrize("prefix", ["-", "+"])
    def test_options(self, bash, prefix):
        completion = assert_complete(bash, f"set {prefix}")
        assert f"{prefix}o" in completion
        assert "+-" not in completion

    @pytest.mark.parametrize("prefix", ["-", "+"])
    def test_o_args(self, bash, prefix):
        completion = assert_complete(bash, f"set {prefix}o ")
        assert any(x.startswith("no") for x in completion)

    @pytest.mark.parametrize("dash,prefix", [["-", "--"], ["-", "+"]])
    def test_options_after_dash_or_dashdash(self, bash, dash, prefix):
        completion = assert_complete(bash, f"set {dash} {prefix}")
        assert not completion



import pytest


class TestJq:
    @pytest.mark.complete("jq ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("jq . ")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete(
        "jq -",
        xfail="! (jq --help 2>&1 || :) | command grep -qF 'options include'",
    )
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("jq --arg ")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("jq --slurpfile ")
    def test_5(self, completion):
        assert not completion

    @pytest.mark.complete("jq --slurpfile foo ")
    def test_6(self, completion):
        assert completion

    @pytest.mark.complete("jq --args ")
    def test_no_completion_after_args(self, completion):
        assert not completion

    @pytest.mark.complete("jq --jsonargs foo ")
    def test_no_completion_after_jsonargs(self, completion):
        assert not completion



import os

import pytest


@pytest.mark.bashcomp(cmd="ether-wake")
class TestEtherWake:
    @pytest.mark.xfail(
        os.environ.get("NETWORK") == "none",
        reason="MAC addresses may be N/A with no networking configured",
    )
    @pytest.mark.complete("ether-wake ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ether-wake -", require_cmd=True)
    def test_2(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(
    cmd="gssdp-device-sniffer",
)
class TestGssdpDeviceSniffer:
    @pytest.mark.complete("gssdp-device-sniffer ")
    def test_basic(self, completion):
        assert not completion

    @pytest.mark.complete("gssdp-device-sniffer -", require_cmd=True)
    def test_options(self, completion):
        assert "--help" in completion



import subprocess

import pytest


class TestKldunload:
    @pytest.mark.complete("kldunload ")
    def test_1(self, completion):
        try:
            subprocess.check_call(
                r"kldstat 2>/dev/null | command grep -q '\.ko$'", shell=True
            )
        except BaseException:
            assert not completion
        else:
            assert completion



import pytest


class TestBts:
    @pytest.mark.complete("bts ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("bts -")
    def test_2(self, completion):
        assert completion



import pytest


class TestMfiUtil:
    @pytest.mark.complete("mfiutil -")
    def test_global_options(self, completion):
        assert completion

    @pytest.mark.complete("mfiutil show ")
    def test_show_subcommand(self, completion):
        assert completion

    @pytest.mark.complete("mfiutil show -")
    def test_options_after_subcommands(self, completion):
        assert not completion

    @pytest.mark.complete("mfiutil -e show dri")
    def test_show_drives_subcommand_with_global_option(self, completion):
        assert completion == ["ves"]

    @pytest.mark.complete("mfiutil locate 1 ")
    def test_locate_subcommand(self, completion):
        assert completion == ["off", "on"]



import pytest


class TestRpm:
    @pytest.mark.complete("rpm ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("rpm -q ", skipif='test ! "$(rpm -qa 2>/dev/null)"')
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("rpm -", require_cmd=True)
    def test_3(self, completion):
        assert completion



import os

import pytest

from conftest import in_container


class TestIfup:
    @pytest.mark.xfail(
        os.environ.get("NETWORK") == "none",
        reason="There won't be any configured interfaces without network",
    )
    @pytest.mark.xfail(in_container(), reason="Probably fails in a container")
    @pytest.mark.complete("ifup ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "ifup -", require_cmd=True, skipif="! ifup --help &>/dev/null"
    )
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("ifup bash-completion ")
    def test_3(self, completion):
        assert not completion



import pytest


@pytest.mark.bashcomp(cmd="munin-node-configure")
class TestMuninNodeConfigure:
    @pytest.mark.complete("munin-node-configure --libdir ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "munin-node-configure -",
        require_cmd=True,
        xfail=(
            "! (munin-node-configure --help 2>&1 || :) "
            "| command grep -q -- '[[:space:]]-'"
        ),
    )
    def test_2(self, completion):
        assert completion



import pytest


class TestPostconf:
    @pytest.mark.complete("postconf -", require_cmd=True)
    def test_1(self, completion):
        assert len(completion) > 1

    # Broken configs may abort output of postconf halfway through, so use
    # something from early output to not trigger false positives because of
    # this. For example, inet_protocols=all but no IPv6 configured:
    # postconf: fatal: parameter inet_interfaces: no local interface found
    #                  for ::1
    # ...and output can be cut off somewhere near lmtp_tls_secur*.
    # ...or be completely missing, so all we can do is to skip.
    @pytest.mark.complete(
        "postconf al", require_cmd=True, xfail="! postconf &>/dev/null"
    )
    def test_2(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(
    cmd="avahi-browse",
)
class TestAvahiBrowse:
    @pytest.mark.complete("avahi-browse --", require_cmd=True)
    def test_options(self, completion):
        assert completion

    @pytest.mark.complete(
        "avahi-browse _",
        require_cmd=True,
        xfail='test ! "$(avahi-browse --dump-db 2>/dev/null)"',
    )
    def test_service_types(self, completion):
        assert completion

    @pytest.mark.complete("avahi-browse -a _")
    def test_no_service_type_with_a(self, completion):
        assert not completion

    @pytest.mark.complete("avahi-browse --dont-fail-in-unset-mode")
    def test_unknown_option(self, completion):
        # Just see that it does not error out
        pass



import pytest


class TestConvert:
    @pytest.mark.complete("convert ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("convert -format ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("convert -", require_cmd=True)
    def test_3(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(
    # On CentOS and Fedora, there's something fishy with consolehelper and
    # /usr/bin/vpnc going on at least when invoked as root; try to invoke the
    # one in /usr/sbin instead.
    pre_cmds=("PATH=/usr/sbin:$PATH",)
)
class TestVpnc:
    @pytest.mark.complete("vpnc -", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest


class TestCppcheck:
    @pytest.mark.complete("cppcheck ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("cppcheck -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("cppcheck -DFOO=BAR ")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("cppcheck -D ")
    def test_4(self, completion):
        assert not completion

    @pytest.mark.complete("cppcheck --enable=al")
    def test_5(self, completion):
        assert completion == "l"

    @pytest.mark.complete("cppcheck --enable=xx,sty")
    def test_6(self, completion):
        assert completion == "le"

    @pytest.mark.complete("cppcheck --enable=xx,yy,sty")
    def test_7(self, completion):
        assert completion == "le"



import pytest

from conftest import (
    assert_bash_exec,
    assert_complete,
    is_bash_type,
    prepare_fixture_dir,
)


@pytest.mark.bashcomp(
    ignore_env=r"^[+-]((BASHOPTS|MANPATH|manpath)=|shopt -. failglob)"
)
class TestMan:
    manpath = "$PWD/man"
    assumed_present = "man"

    @pytest.fixture
    def colonpath(self, request, bash):
        try:
            assert_bash_exec(bash, "uname -s 2>&1 | grep -qiF cygwin")
        except AssertionError:
            pass
        else:
            pytest.skip("Cygwin doesn't like paths with colons")
            return
        tmpdir, _, _ = prepare_fixture_dir(
            request,
            files=["man/man3/Bash::Completion.3pm.gz"],
            dirs=["man", "man/man3"],
        )
        return tmpdir

    @pytest.mark.complete(
        "man bash-completion-testcas",
        env=dict(MANPATH=manpath),
        require_cmd=True,
    )
    def test_1(self, completion):
        assert completion == "e"

    @pytest.mark.complete("man man1/f", cwd="man", env=dict(MANPATH=manpath))
    def test_2(self, completion):
        assert completion == "oo.1"

    @pytest.mark.complete("man man/", cwd="man", env=dict(MANPATH=manpath))
    def test_3(self, completion):
        assert completion == "quux.8"

    @pytest.mark.complete(
        "man %s" % assumed_present,
        cwd="shared/empty_dir",
        env=dict(MANPATH=manpath),
    )
    def test_4(self, completion):
        """
        Assumed present should not be completed complete when there's no
        leading/trailing colon in $MANPATH.
        """
        assert not completion

    @pytest.mark.complete(
        "man %s" % assumed_present,
        require_cmd=True,
        cwd="shared/empty_dir",
        env=dict(MANPATH="%s:" % manpath),
    )
    def test_5(self, completion):
        """Trailing colon appends system man path."""
        assert completion

    @pytest.mark.complete(
        "man bash-completion-testcas",
        require_cmd=True,
        env=dict(MANPATH="%s:" % manpath),
    )
    def test_6(self, completion):
        assert completion == "e"

    @pytest.mark.complete(
        "man %s" % assumed_present,
        require_cmd=True,
        cwd="shared/empty_dir",
        env=dict(MANPATH=":%s" % manpath),
    )
    def test_7(self, completion):
        """Leading colon prepends system man path."""
        assert completion

    @pytest.mark.complete(
        "man bash-completion-testcas",
        require_cmd=True,
        env=dict(MANPATH=":%s" % manpath),
    )
    def test_8(self, completion):
        assert completion == "e"

    @pytest.mark.complete(
        "man %s" % assumed_present,
        require_cmd=True,
        cwd="shared/empty_dir",
        shopt=dict(failglob=True),
    )
    def test_9(self, bash, completion):
        assert self.assumed_present in completion

    def test_10(self, request, bash, colonpath):
        if not is_bash_type(bash, "man"):
            pytest.skip("Command not found")
        completion = assert_complete(
            bash,
            "man Bash::C",
            env=dict(MANPATH="%s:%s/man" % (TestMan.manpath, colonpath)),
        )
        assert completion == "ompletion"

    @pytest.mark.complete("man -", require_cmd=True)
    def test_11(self, completion):
        assert completion

    @pytest.mark.complete("man -S 1", require_cmd=True)
    def test_delimited_first(self, completion):
        # just appends space
        assert not completion
        assert completion.endswith(" ")

    @pytest.mark.complete("man -S 1:", require_cmd=True)
    def test_delimited_after_delimiter(self, completion):
        assert completion
        assert "1" not in completion

    @pytest.mark.complete("man -S 1:2", require_cmd=True)
    def test_delimited_later(self, completion):
        # just appends space
        assert not completion
        assert completion.endswith(" ")

    @pytest.mark.complete("man -S 1:1", require_cmd=True)
    def test_delimited_deduplication(self, completion):
        # no completion, no space appended
        assert not completion
        assert not completion.endswith(" ")

    @pytest.mark.complete(
        "man bash-completion-zstd-testcas",
        env=dict(MANPATH=manpath),
        require_cmd=True,
    )
    def test_zstd_arbitrary_sectsuffix(self, completion):
        assert completion == "e"

    @pytest.mark.complete(
        "man bash-completion-testcas",
        env=dict(MANPATH="'$(echo malicious code >/dev/tty)'"),
    )
    def test_manpath_code_injection(self, completion):
        # no completion, no space appended
        assert not completion



import pytest


@pytest.mark.bashcomp(cmd="xvfb-run")
class TestXvfbRun:
    @pytest.mark.complete("xvfb-run ")
    def test_no_args(self, completion):
        assert any(x in completion for x in ("bash", "xvfb-run"))

    @pytest.mark.complete("xvfb-run -", require_cmd=True)
    def test_options(self, completion):
        assert completion



import pytest

from conftest import assert_bash_exec


@pytest.mark.bashcomp(cmd=None, ignore_env=r"^\+COMPREPLY=")
class TestUnitCompgenTilde:
    def test_1(self, bash):
        assert_bash_exec(bash, "! _comp_compgen_tilde >/dev/null")

    def test_2(self, bash):
        """Test environment non-pollution, detected at teardown."""
        assert_bash_exec(
            bash,
            'foo() { local aa="~"; _comp_compgen -c "$aa" tilde; }; foo; unset -f foo',
        )

    def test_3(self, bash):
        """Test for https://bugs.debian.org/766163"""
        assert_bash_exec(bash, "! _comp_compgen -c ~-o tilde")

    def _test_part_full(self, bash, part, full):
        res = (
            assert_bash_exec(
                bash,
                '_comp_compgen -c "~%s" tilde; echo "${COMPREPLY[@]}"' % part,
                want_output=True,
            )
            .strip()
            .split()
        )
        assert res
        assert res[0] == "~%s" % full

    def test_4(self, bash, part_full_user):
        """~full should complete to ~full unmodified."""
        _, full = part_full_user
        self._test_part_full(bash, full, full)

    def test_5(self, bash, part_full_user):
        """~part should complete to ~full."""
        part, full = part_full_user
        self._test_part_full(bash, part, full)



import pytest


class TestLvm:
    @pytest.mark.complete("lvm pv")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "lvm lvcreate --",
        require_cmd=True,
        xfail="! lvm lvcreate --help &>/dev/null",
    )
    def test_subcommand_options(self, completion):
        assert completion



import pytest


@pytest.mark.bashcomp(cmd="apt-mark")
class TestAptMark:
    @pytest.mark.complete("apt-mark ")
    def test_1(self, completion):
        assert all(
            x in completion
            for x in (
                "auto manual remove showinstall showremove "
                "hold minimize-manual showauto showmanual unhold install "
                "purge showhold showpurge"
            ).split()
        )

    @pytest.mark.complete("apt-mark minimize-manual ")
    def test_2(self, completion):
        assert not completion

    @pytest.mark.complete("apt-mark --file=", cwd="dpkg")
    def test_3(self, completion):
        assert (
            completion
            == "bash-completion-test-nonsubject.txt bash-completion-test-subject.deb".split()
        )

    @pytest.mark.complete("apt-mark --config-file ", cwd="apt-mark")
    def test_4(self, completion):
        assert completion == "example.conf"

    @pytest.mark.complete("apt-mark --option ")
    def test_5(self, completion):
        assert not completion

    @pytest.mark.complete("apt-mark --dont-fail-in-unset-mode")
    def test_unknown_option(self, completion):
        # Just see that it does not error out
        pass



import os

import pytest


class TestDict:
    @pytest.mark.complete("dict -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.xfail(
        os.environ.get("NETWORK") == "none",
        reason="The database list is unavailable without network",
    )
    @pytest.mark.complete("dict --database ", require_cmd=True)
    def test_database(self, completion):
        # Ensure the directory name "_comp_load/" not generated because
        # filenames in the current dictory (i.e., test/fixtures) are generated
        # by "-o default" when "_comp_cmd_dict" fails to generate any
        # completions.
        assert completion and "_comp_load/" not in completion

    @pytest.mark.xfail(
        os.environ.get("NETWORK") == "none",
        reason="The database list is unavailable without network",
    )
    @pytest.mark.complete(
        "dict -h dict.org --database ", require_cmd=True, env=dict(IFS="")
    )
    def test_database_IFS(self, completion):
        assert completion and "_comp_load/" not in completion



import pytest


@pytest.mark.bashcomp(cmd="btdownloadcurses.py")
class TestBtdownloadcursesPy:
    @pytest.mark.complete("btdownloadcurses.py ")
    def test_1(self, completion):
        assert completion



import pytest


class TestBsdtar:
    @pytest.mark.complete("bsdtar xf bsdtar/")
    def test_readable_archives(self, completion):
        assert completion == "test.pax test.rar".split()

    @pytest.mark.complete("bsdtar uf bsdtar/")
    def test_writable_archives(self, completion):
        assert completion == "test.pax test.shar".split()



import pytest

from conftest import assert_bash_exec, bash_env_saved


@pytest.mark.bashcomp(
    cmd=None,
    cwd="_filedir",
    ignore_env=r"^\+declare -f (dump_array|__tester)$",
)
class TestExpandGlob:
    @pytest.fixture(scope="class")
    def functions(self, bash):
        assert_bash_exec(
            bash,
            "dump_array() { ((${#arr[@]})) && printf '<%s>' \"${arr[@]}\"; echo; }",
        )
        assert_bash_exec(
            bash,
            '__tester() { local LC_ALL= LC_COLLATE=C arr; _comp_expand_glob arr "$@";dump_array; }',
        )

    def test_match_all(self, bash, functions):
        output = assert_bash_exec(bash, "__tester '*'", want_output=True)
        assert (
            output.strip()
            == "<a b><a$b><a&b><a'b><ab><aé><brackets><dotdot><ext>"
        )

    def test_match_pattern(self, bash, functions):
        output = assert_bash_exec(bash, "__tester 'a*'", want_output=True)
        assert output.strip() == "<a b><a$b><a&b><a'b><ab><aé>"

    def test_match_unmatched(self, bash, functions):
        output = assert_bash_exec(
            bash, "__tester 'unmatched-*'", want_output=True
        )
        assert output.strip() == ""

    def test_match_multiple_words(self, bash, functions):
        output = assert_bash_exec(bash, "__tester 'b* e*'", want_output=True)
        assert output.strip() == "<brackets><ext>"

    def test_match_brace_expansion(self, bash, functions):
        output = assert_bash_exec(
            bash, "__tester 'brac{ket,unmatched}*'", want_output=True
        )
        assert output.strip() == "<brackets>"

    def test_protect_from_noglob(self, bash, functions):
        with bash_env_saved(bash, functions) as bash_env:
            bash_env.set("noglob", True)
            output = assert_bash_exec(bash, "__tester 'a*'", want_output=True)
            assert output.strip() == "<a b><a$b><a&b><a'b><ab><aé>"

    def test_protect_from_failglob(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("failglob", True)
            output = assert_bash_exec(
                bash, "__tester 'unmatched-*'", want_output=True
            )
            assert output.strip() == ""

    def test_protect_from_nullglob(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("nullglob", False)
            output = assert_bash_exec(
                bash, "__tester 'unmatched-*'", want_output=True
            )
            assert output.strip() == ""

    def test_protect_from_dotglob(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            bash_env.shopt("dotglob", True)
            output = assert_bash_exec(
                bash, "__tester 'ext/foo/*'", want_output=True
            )
            assert output.strip() == ""

    def test_protect_from_GLOBIGNORE(self, bash, functions):
        with bash_env_saved(bash) as bash_env:
            # Note: dotglob is changed by GLOBIGNORE
            bash_env.save_shopt("dotglob")
            bash_env.write_variable("GLOBIGNORE", "*")
            output = assert_bash_exec(bash, "__tester 'a*'", want_output=True)
            assert output.strip() == "<a b><a$b><a&b><a'b><ab><aé>"



import pytest


class TestPylint:
    @pytest.mark.complete("pylint --v", require_longopt=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("pylint --confidence=HIGH,")
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("pylint --help-msg=", require_longopt=True)
    def test_all_message_ids(self, completion):
        assert any("-" in x for x in completion)

    @pytest.mark.complete("pylint --disable=", require_longopt=True)
    def test_enabled_message_ids(self, completion):
        assert any("-" in x for x in completion)

    @pytest.mark.complete("pylint --enable=foo,", require_longopt=True)
    def test_disabled_message_ids(self, completion):
        assert any("-" in x for x in completion)



import pytest


class TestSs:
    @pytest.mark.complete("ss -", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ss -A ", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete("ss -A foo,", require_cmd=True)
    def test_3(self, completion):
        assert completion



import pytest

from conftest import assert_bash_exec, bash_env_saved


@pytest.mark.bashcomp(cmd=None)
class TestUnitXfunc:
    def test_1(self, bash):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable(
                "BASH_COMPLETION_USER_DIR", "$PWD/_comp_xfunc", quote=False
            )

            # test precondition
            assert_bash_exec(
                bash,
                "! declare -F _comp_xfunc_xfunc_test1_utility1 &>/dev/null",
            )

            # first invocation (completion/xfunc-test1 is sourced)
            output = assert_bash_exec(
                bash,
                "_comp_xfunc xfunc-test1 utility1 'a b' cde fgh",
                want_output=True,
            )
            assert output.strip() == "util1[<a b><cde><fgh>]"

            # test precondition
            assert_bash_exec(
                bash, "declare -F _comp_xfunc_xfunc_test1_utility1 &>/dev/null"
            )

            # second invocation (completion/xfunc-test1 is not sourced)
            output = assert_bash_exec(
                bash,
                "_comp_xfunc xfunc-test1 utility1 'a b' cde fgh",
                want_output=True,
            )
            assert output.strip() == "util1[<a b><cde><fgh>]"

    def test_2(self, bash):
        with bash_env_saved(bash) as bash_env:
            bash_env.write_variable(
                "BASH_COMPLETION_USER_DIR", "$PWD/_comp_xfunc", quote=False
            )

            # test precondition
            assert_bash_exec(
                bash, "! declare -F _comp_xfunc_non_standard_name &>/dev/null"
            )

            # first invocation (completion/xfunc-test2 is sourced)
            output = assert_bash_exec(
                bash,
                "_comp_xfunc xfunc-test2 _comp_xfunc_non_standard_name 'a b' cde fgh",
                want_output=True,
            )
            assert output.strip() == "util2[<a b><cde><fgh>]"

            # test precondition
            assert_bash_exec(
                bash, "declare -F _comp_xfunc_non_standard_name &>/dev/null"
            )

            # second invocation (completion/xfunc-test2 is not sourced)
            output = assert_bash_exec(
                bash,
                "_comp_xfunc xfunc-test2 _comp_xfunc_non_standard_name 'a b' cde fgh",
                want_output=True,
            )
            assert output.strip() == "util2[<a b><cde><fgh>]"



import pytest


class TestCcze:
    @pytest.mark.complete("ccze ", require_cmd=True)
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("ccze -? ")
    def test_2(self, completion):
        assert not completion

    @pytest.mark.complete("ccze -o ")
    def test_3(self, completion):
        assert completion

    @pytest.mark.complete("ccze --plugin=", require_cmd=True)
    def test_4(self, completion):
        assert completion



import os

import pytest


@pytest.mark.bashcomp(cmd="pkgconf")
class TestPkgconf:
    @pytest.mark.complete("pkgconf ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("pkgconf -", require_cmd=True)
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete(
        "pkgconf %s/bash-completion.pc --variable="
        % os.getenv("ABS_TOP_BUILDDIR", "../.."),
        require_cmd=True,
    )
    def test_variable(self, completion):
        assert "completionsdir" in completion



import pytest


@pytest.mark.bashcomp(cmd="freeciv-server")
class TestFreecivServer:
    @pytest.mark.complete("freeciv-server -", require_cmd=True)
    def test_1(self, completion):
        assert completion



import pytest

from conftest import assert_complete, partialize


@pytest.mark.bashcomp(pre_cmds=("HOME=$PWD",))
class TestXhost:
    @pytest.mark.parametrize("prefix", ["+", "-", ""])
    def test_hosts(self, bash, hosts, prefix):
        completion = assert_complete(bash, "xhost %s" % prefix)
        assert completion == [f"{prefix}{x}" for x in hosts]

    @pytest.mark.parametrize("prefix", ["+", "-", ""])
    def test_partial_hosts(self, bash, hosts, prefix):
        first_char, partial_hosts = partialize(bash, hosts)
        completion = assert_complete(bash, f"xhost {prefix}{first_char}")
        if len(completion) == 1:
            assert completion == partial_hosts[0][1:]
        else:
            assert completion == sorted(f"{prefix}{x}" for x in partial_hosts)



import sys

import pytest


class TestDmesg:
    @pytest.mark.complete("dmesg -", require_cmd=True)
    def test_1(self, completion):
        if sys.platform == "darwin":
            assert not completion  # takes no options
        else:
            assert completion



import pytest


@pytest.mark.bashcomp(cmd="ssh-add")
class TestSshAdd:
    @pytest.mark.complete("ssh-add ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete(
        "ssh-add -",
        require_cmd=True,
        xfail="ssh-add --help 2>&1 | "
        "command grep -qiF 'Could not open a connection'",
    )
    def test_2(self, completion):
        assert completion

    @pytest.mark.complete(
        "ssh-add -",
        require_cmd=True,
        xfail="ssh-add --help 2>&1 | "
        "command grep -qiF 'Could not open a connection'",
        shopt=dict(failglob=True),
    )
    def test_2_failglob(self, completion):
        assert completion

    @pytest.mark.complete(
        "ssh-add -",
        require_cmd=True,
        xfail="ssh-add --help 2>&1 | "
        "command grep -qiF 'Could not open a connection'",
        shopt=dict(nullglob=True),
    )
    def test_2_nullglob(self, completion):
        assert completion



import pytest


class TestNproc:
    @pytest.mark.complete("nproc ")
    def test_1(self, completion):
        assert not completion

    @pytest.mark.complete("nproc --", require_longopt=True)
    def test_2(self, completion):
        assert completion



import sys

import pytest


@pytest.mark.bashcomp(cmd="invoke-rc.d")
class TestInvokeRcD:
    @pytest.mark.xfail(
        sys.platform == "darwin",
        reason="Service completion not available on macOS",
    )
    @pytest.mark.complete("invoke-rc.d ")
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("invoke-rc.d --no-fallback --")
    def test_2(self, completion):
        """Test already specified option is not offered."""
        if sys.platform != "darwin":  # no service completion
            assert completion
        assert "--no-fallback" not in completion



import pytest


class TestVgreduce:
    @pytest.mark.complete(
        "vgreduce -", require_cmd=True, xfail="! vgreduce --help &>/dev/null"
    )
    def test_1(self, completion):
        assert completion



import pytest


class TestVgcreate:
    @pytest.mark.complete(
        "vgcreate -", require_cmd=True, xfail="! vgcreate --help &>/dev/null"
    )
    def test_1(self, completion):
        assert completion

    @pytest.mark.complete("vgcreate __does_not_exist__", require_cmd=True)
    def test_2(self, completion):
        assert not completion
